/* 
 * Alertsx.h
 *
 * Copyright  1995 by Michael D. Trent; all rights reserved
 */

#include "Alertsx.h"
#include "MMR.h"
//#include "butil.h"

#define kEnterKey		3
#define kReturnKey		13
#define kEscapeKey		27

#define kErrorBaseID	128
#define kErrorAlertID	(kErrorBaseID + fatal)

#define kErrorStrings	128

//void PStringCat(Str255 a, Str255 b);

/* DoErr
 * This is a basic error routine, tossing up a StopAlert displaying a 
 * single verbose string and an error number.  If fatal is set to true,
 * the procedure will call ExitToShell();
 *
 * The format of the alert resource is a basic alert; '^0' and '^1' 
 * param text characters; and an OK button, which must have ID 1.
 */
// void DoErr (Str255 errorString, OSErr err, short fatal)
void DoErr (short errorID,OSErr err, Str255 file, short fatal)
{
	Str255 errorString,errNumString, scratch;
	
	switch (errorID) {
	case 4:
		GetIndString(errorString, kErrorStrings, 8); // "The file "
		GetIndString(scratch, kErrorStrings, 4); // " could not ..."
		PStringCat(errorString,file);
		PStringCat(errorString,scratch);
		break;
	//case 5:
	case 9:
	case 10:
	case 11:
	case 12:
		GetIndString(errorString, kErrorStrings, errorID); // "The <appleevent> "
		GetIndString(scratch, kErrorStrings, 5); // " AppleEvent ..."
		PStringCat(errorString,scratch);
		break;
	default:
		GetIndString(errorString, kErrorStrings, errorID);
	}
		
	NumToString((long) err, errNumString);
	
	ParamText (errorString, errNumString, "\p", "\p");
	
	StopAlert (kErrorAlertID, NewModalFilterProc(AlertKeyProc));
	
	if(fatal)
		ExitToShell();
}

/* DoError
 * Show an error alert dialog if a system error occurs.
 */
void DoError (Str255 errorString, OSErr err, Boolean fatal)
{
	Str255 errNoString;
	
	NumToString(err,errNoString);
	ParamText (errorString, errNoString, "\p", "\p");
	
	StopAlert (kErrorAlertID, nil);
		
	if(fatal)
		ExitToShell();
}



/* AlertKeyProc 
 * This AlertProc automatically processes the standard keystrokes: Return/Enter = OK,
 * Escape = Cancel.  Also, if a user presses a key corresponding to the first character
 * of a button (including OK or Cancel), that button will be selected.
 *
 * This routine assumes that the second DITL item is a push button, then it
 * is the "Cancel" button.  If there is no Cancel button, but there are more than
 * one push buttons and a DITL item w/ ID # 2 exists, it must NOT be a push button.
 *
 * Pre: Either a Cancel button exists, or DITL item # 2 is not a push button.
 * Post: Key strokes were handled appropriately.
 */
/* Routine updated to recognize command-'.' as Cancel
 * 4 MAR 95
 */
/* Fixed bug in forloop (i = 0; i<num ... (i=1; i<=num ..))
 * 24 APR 95
 */
/* Added Standard filter proc - not PPC friendly?
 * 23 Oct 95
 */
/* Added key-control for Balloon Help
 * 1/31/96
 */
pascal Boolean AlertKeyProc(DialogPtr d, EventRecord *e, short *itemHit)
{
	char 	theChar;
	short	num, i, iType;
	Handle	iHandle;
	Rect	iRect;
	Str255	iText;
	
	ModalFilterUPP	stdFilterProc;
	Boolean ret;
	OSErr	err;

	switch(e->what) {
	case keyDown:
	case autoKey:
		theChar = (e->message & charCodeMask);
		
		// MDT 1/31/96 turn balloon help on/off
		if (theChar == 0x05) {
			(void) HMSetBalloons(!HMGetBalloons()); 
			return false;
		}
		
		if (theChar == kReturnKey || theChar == kEnterKey) {
			GetDItem(d, 1, &iType, &iHandle, &iRect);
			FlashButton((ControlHandle)iHandle);
			
			*itemHit = 1;
			return (true);
		} else if ( (theChar == kEscapeKey) ||
				   ((theChar == '.') && (e->modifiers & cmdKey) ))  {
			GetDItem(d, 2, &iType, &iHandle, &iRect);
			if (iType == ctrlItem + btnCtrl) {		/* If there is a Button */
				FlashButton((ControlHandle)iHandle);
			
				*itemHit = 2;
				return (true);
			} else {
				return (false);
			}
		} else {
			//if ((e->modifiers & cmdKey)) { /* Only work if command key is down */
			num = CountDITL(d);
			for (i=1; i<=num; i++) {
				GetDItem(d, i, &iType, &iHandle, &iRect);
				if (iType == ctrlItem + btnCtrl) {
					GetCTitle((ControlHandle)iHandle, iText);
					if (theChar == iText[1] || theChar == (char)(iText[1] + 32)){
						FlashButton((ControlHandle)iHandle);
						*itemHit = i;
						return (true);
					}	// if
				}	// if
			}	// for
			//}	// if
		} // else
		break;
	default:
		;
	}	// switch
	//return (false);
	
	// cal the standard filter proc
	return StdFilterProc(d,e,itemHit);
	
	//err = GetStdFilterProc(&stdFilterProc);
	//if (err == noErr) ret = (stdFilterProc)(d,e,itemHit);
	//else ret = false;
	
	//return false;
}

/* DrawDefaultButton
 * OBSOLETE!!!!!!!!!!!!!!!!
 * Draws that silly bold rounded rectangle around the default button in a Dialog.
 *
 * Pre: Default button is ID 1
 * Post: The rect was drawn, all port and pen state info is restored 
 */
void DrawDefaultButton (DialogPtr d)
{
	Handle item;
	short itemType;
	Rect r;
	PenState oldPen;
	GrafPtr oldPort;

	GetPort(&oldPort);
	SetPort((WindowPtr)d);
	GetPenState(&oldPen);
	GetDItem(d, 1, &itemType, &item, &r);
	PenSize(3, 3);
	InsetRect(&r, -4, -4);
	FrameRoundRect(&r, 16, 16);
	SetPenState(&oldPen);
	SetPort(oldPort);
}

/* FlashButton
 * Given a handle to a push button, FlashButton will select the button for a pre-set
 * duration of time, and then un-select said button.
 */
#define kTickWait		5

void FlashButton(ControlHandle control)
{
	long 	waitTill;
	
	waitTill = TickCount() + kTickWait;

	HiliteControl(control,1);
	
	while (waitTill>TickCount());
	
	HiliteControl(control,0);
}
