/* Currently case sensitive. In the future (maybe), will default to case-insensitive,
 * with an option for case-sensitive lists.
 */

#include "AlphaList.h"

#define kHeadNode		(unsigned char)1
#define kTailNode		(unsigned char)2
//#define kCaseSensitive	(unsigned char)1

//Errors
#define kMemoryError	-1
#define kNullString		-2
#define kNodeNotFound	-3

AlphaLinkedList NewAlphaLinkedList(void)
//AlphaLinkedList NewAlphaLinkedList(Boolean caseSensitive)
{
	AlphaLinkPtr head,tail;
	
	tail = (AlphaLinkPtr) NewPtr(sizeof(AlphaLink));
	if (!tail) goto foo;
	
	tail->data = (void *) 0; // for no reason
	tail->key[0] = 0; tail->key[1] = kTailNode; 
	tail->next = tail;
	
	head = (AlphaLinkPtr) NewPtr(sizeof(AlphaLink));
	if (!head) goto foo;
	
	head->data = (void *) 0; // 0 nodes in the list
	head->key[0] = 0; head->key[1] = kHeadNode;
	head->next = tail;
	
	return head;

foo:
	if (tail) DisposePtr((Ptr)tail);
	if (head) DisposePtr((Ptr)head); // unreachable code.

	return 0L;
}

// warning DisposeAlphaLinkedList will orphan memory! We shouldn't go
// deleting void*s at random ... 
void DisposeAlphaLinkedList(AlphaLinkedList *list)
{
	AlphaLinkPtr	cur,del = *list;
	
	cur = (*list)->next;
	DisposePtr((Ptr)del); // dispose head
	
	while (cur->key[0] != 0) { // while cur != z
		del = cur;
		cur = cur->next;
		DisposePtr((Ptr)del);
	}
	DisposePtr((Ptr)cur); // dispose z

	*list = nil; // set list to nil
}

short	AddAlphaItem(AlphaLinkedList list, Str255 sortKey, void *data)
{
	AlphaLinkPtr	cur = list, newLink;
	
	if (sortKey[0] == 0) return kNullString;
	
	newLink = (AlphaLinkPtr) NewPtr(sizeof(AlphaLink));
	if (!newLink) return kMemoryError;
	
	newLink->data = data;
	BlockMove(sortKey,newLink->key,sortKey[0]);
	
	while ((cur->next->key[0] != 0) // while cur->next != Z and ...
		&& (IUCompString(sortKey, cur->next->key) > 0)) { // ... sortKey > next->key
		cur = cur->next;
	}
	
	newLink->next = cur->next;
	cur->next = newLink;
	
	return 0;
}

short RemoveAlphaItem(AlphaLinkedList list, Str255 sortKey)
{
	AlphaLinkPtr	cur = list,del ;
	
	if (sortKey[0] == 0) return kNullString;
	
	while ((cur->next->key[0] != 0) // while cur->next != Z and ...
		&& (! EqualString(sortKey, cur->next->key, true, true) )) { // ... sortKey != cur->next->key
		cur = cur->next;
	}
	
	if (cur->next->key[0] == 0) return kNodeNotFound;
	
	del = cur->next;
	cur->next = del->next;
	
	DisposePtr((Ptr)del);
	
	return 0;
}

void *GetAlphaItem(AlphaLinkedList list, Str255 sortKey)
{
	AlphaLinkPtr	cur = list->next;
	
	//if (sortKey[0] == 0) return kNullString;
	if (sortKey[0] == 0) return nil;
	
	while ((cur->key[0] != 0) // while cur != Z and ...
		&& (! EqualString(sortKey, cur->key, true, true) )) { // ... sortKey != cur->key
		cur = cur->next;
	} 
	
	//if (cur->key[0] == 0) return kNodeNotFound; // if cur == Z
	if (cur->key[0] == 0) return nil; // if cur == Z
	
	return cur->data;
}

