/*
 * AppleEvents.c
 */

#include "MMR.h"
#include "Alertsx.h"
#include "AppleEventsx.h"
#include "Scen Window.h"
#include "Scenario.h"
#include "Map Window.h"
#include "TileMenuCommon.h"
#include "Eventsx.h"
#include "Menusx.h"
#include "About.h"

Boolean gOpenAppEvent = false;

pascal OSErr DoAEOpenApp (AppleEvent *theEvent, AppleEvent *reply, long refCon);
pascal OSErr DoAEOpenDoc (AppleEvent *theEvent, AppleEvent *reply, long refCon);
pascal OSErr DoAEPrintDoc (AppleEvent *theEvent, AppleEvent *reply, long refCon);
pascal OSErr DoAEQuit (AppleEvent *theEvent, AppleEvent *reply, long refCon);

void InitAppleEvents (void)
{
	OSErr			err;
	
	/* Since we don't rely on Apple Events for much of anything, errors
	   are not fatal here */
	err = AEInstallEventHandler (kCoreEventClass, kAEOpenApplication, NewAEEventHandlerProc( DoAEOpenApp ), 0, false);
	if (err != noErr) DoError("\pCouldn't set AEOpenApplication AppleEvent!",err,kNotFatal);
	err = AEInstallEventHandler (kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerProc( DoAEOpenDoc ), 0, false);
	if (err != noErr) DoError("\pCouldn't set AEOpenDocuments AppleEvent!",err,kNotFatal);
	err = AEInstallEventHandler (kCoreEventClass, kAEPrintDocuments, NewAEEventHandlerProc( DoAEPrintDoc ), 0, false);
	if (err != noErr) DoError("\pCouldn't set AEPrintDocuments AppleEvent!",err,kNotFatal);
	err = AEInstallEventHandler (kCoreEventClass, kAEQuitApplication, NewAEEventHandlerProc( DoAEQuit ), 0, false);
	if (err != noErr) DoError("\pCouldn't set AEQuitApplication AppleEvent!",err,kNotFatal);
	return;
}

/**
 **  APPLE EVENTS
 **/
 
pascal OSErr DoAEOpenApp (AppleEvent *theEvent, AppleEvent *reply, long refCon)
{
	long wait = TickCount() + (2*60);
	EventRecord theEvent2;
	
	while (wait > TickCount()) {
		if(WaitNextEvent(mouseDown+mouseUp+keyDown+autoKey, &theEvent2,0,nil)) break;
	}
	CloseAbout();
	UpdateFileMenu();

	gOpenAppEvent = true;
	
	return noErr;
}

/* OpenDoc
 */
pascal OSErr DoAEOpenDoc (AppleEvent *theEvent, AppleEvent *reply, long refCon)
{
	FSSpec 		myFSS;
	AEDescList	docList;
	OSErr		err;
	long		index, itemsInList;
	Size		actualSize;
	AEKeyword	keywd;
	DescType	returnedType;
	WindowPtr	w;
	
	err = AEGetParamDesc(theEvent, keyDirectObject, typeAEList, &docList);
	if (err != noErr) {
		//DoError(kAppleEventError,err,"\p",kNotFatal);
		goto foo;
	}
	
	err = AECountItems(&docList,&itemsInList);
	if (err != noErr) {
		//DoError(kAppleEventError,err,"\p",kNotFatal);
		goto foo;
	}
	
	index = 1; // we only open one doc, not all of them ... :(
	err = AEGetNthPtr(&docList, index, typeFSS, &keywd, &returnedType, &myFSS, 
				sizeof(myFSS), &actualSize);
	if (err != noErr) {
		//DoError(kAppleEventError,err,"\p",kNotFatal);
		goto foo;
	}
		
	// BEGIN: LOAD ULTIMA FILE
	if (gScenWindow) {
		if (CloseScenWindow()) {
			//CloseAllMapWindows();
			ForEachMapWindowDo(CloseMapWindow);
		} else {
			goto foo;
		}
	}
	
	(void) OpenScenarioFile(&myFSS,false);
	SetTMTiles(gScenario.tiles);
	CheckTMItem(1);
	w = NewScenWindow();
	// END: LOAD ULTIMA FILE	
		
	err = AEDisposeDesc(&docList);
	err = noErr; // ignore last error

foo:
	if (!gOpenAppEvent) {
		CloseAbout();
		gOpenAppEvent = true; // we set this here so that if the program starts with an OpenDoc
		                      // event, it doesn't quit, then the user selects "Quit when.."
		                      // the program won't quit the next time they drag a file on to it
		                      // (unless they quit and restart)
	}
	
	UpdateFileMenu();
	return err;
}


pascal OSErr DoAEPrintDoc (AppleEvent *theEvent, AppleEvent *reply, long refCon)
{
	if (!gOpenAppEvent) gDone = true; // quit if we were started with an open app event.

	return noErr;
}

pascal OSErr DoAEQuit (AppleEvent *theEvent, AppleEvent *reply, long refCon)
{
	gDone = true;
	
	return noErr;
}