/*
 * Locations.c
 *
 * Routines for managing the locations of castles, townes, dungeons, and moon gates.
 * 5/26/96
 */

#include "Locations.h"
#include "Scenario.h"

#define kMaxMoongates	8
#define kMaxCiDu		19

#define kMaxMGYOffset	8
#define kMaxCiDuYOffset	0x20

short gLocResID[] = { 4,4,0 }; // cities/townes, dungeons, moongates

short DungeonOrTowne(short locID);

short FindLocationByPosition(short locType, Point position)
{
	Handle h;
	short i;
	Point pt;
	short ret = -1;
	
	if ((locType < kLocCity) || (locType > kLocMoongate)) return -1;
	
	h = gScenario.misc[gLocResID[locType]].misc;
	switch (locType) {
	case kLocCity:
	case kLocDungeon:
		for (i=0; i<kMaxCiDu; i++) {
			pt.h=(*h)[i ];
			pt.v=(*h)[i + kMaxCiDuYOffset];
			if ((EqualPt(pt,position)) && (locType == DungeonOrTowne(i))) {
				ret = i;
				break; // for
			}
		}
		break; // case
	case kLocMoongate:
		for (i=0; i<kMaxMoongates; i++) {
			pt.h=(*h)[i];
			pt.v=(*h)[i+kMaxMGYOffset];
			if (EqualPt(pt,position)) {
				ret = i;
				break; // for
			}
		}
		break; // case
	}
	
	return ret;
}

void FindLocationByID(short locType, Point *position, short locID)
{
	Handle h;
	short offset;
	
	if ((locType < kLocCity) || (locType > kLocMoongate)) return;
	
	offset = (locType == kLocMoongate) ? kMaxMGYOffset : kMaxCiDuYOffset;
	h = gScenario.misc[gLocResID[locType]].misc;
	position->h=(*h)[locID];
	position->v=(*h)[locID+offset];
}

void SetLocation(short locType, short locID, Point position)
{
	Handle h;
	short offset;
	
	if ((locType < kLocCity) || (locType > kLocMoongate)) return;
	
	offset = (locType == kLocMoongate) ? kMaxMGYOffset : kMaxCiDuYOffset;
	h = gScenario.misc[gLocResID[locType]].misc;
	(*h[locID]) = position.h;
	(*h[locID+offset]) = position.v;

	// set for the 500 range too...
	h = gScenario.misc[gLocResID[locType]+6].misc;
	(*h[locID]) = position.h;
	(*h[locID+offset]) = position.v;
}

short DungeonOrTowne(short locID)
{
	if (locID < 12) return kLocCity;
	
	return kLocDungeon;
}