/*
 * Mike's Misc Routines
 *
 * Lovingly implementation independant.
 */

/* Revised: 11/1/95 */

#include <fp.h>
#include "MMR.h"

#include <QDOffscreen.h>

/* Private Prototypes */
void TagZero(Str255 s);

/* 
 * Quick Draw 
 */


/* MakeGWorld
 * Makes a GWorld of a specified dimension and depth. Returns OSErr.
 */

OSErr MakeGWorld(GWorldPtr *w, Rect *r, int depth)
{
	OSErr err;
	
	err = NewGWorld(w, depth, r, 0L, 0L, 0);
	if (err != noErr) *w = nil; 
	
	return (err);
}

/* DrawMyPict
 * Draws a PicHandle into a GWorldPtr. It handles GWorld locking and unlocking.
 * Pre: Both the world and the pict exist.
 * Post: the picture was drawn into the world, at 0,0
 */
void DrawMyPict (GWorldPtr world, PicHandle pict)
{
	Rect r = (*pict)->picFrame;
	GWorldPtr oldGW;
	GDHandle oldGD;
	
	OffsetRect(&r, -r.left, -r.top);
	GetGWorld(&oldGW, &oldGD);
	LockPixels(world->portPixMap);
	SetGWorld(world, 0L);
	
	DrawPicture (pict, &r);
	SetGWorld(oldGW, oldGD);
	UnlockPixels(world->portPixMap);
}

/* MyDrawGrowIcon
 * Replaces DrawGrowIcon. It draws only the grow icon, clipping out the scrollbar
 * lines. This avoids a few bugs (features?) of QuickDraw that aren't completely
 * desireable.
 */
void MyDrawGrowIcon(WindowPtr w)
{
	Rect gi;
	RgnHandle rgn, saveRgn;
	
	if (!w) return;
	SetPort(w); // REMOVE AND DIE!!
	
	saveRgn = NewRgn();
	rgn = NewRgn();
	
	GetClip(saveRgn);
	OpenRgn();
	gi = w->portRect;
	SetRect(&gi, gi.right - 15, gi.bottom - 15, gi.right, gi.bottom);
	FrameRect(&gi);
	CloseRgn(rgn);
	
	SetClip(rgn);
	DrawGrowIcon(w);
	SetClip(saveRgn);
	
	DisposeRgn(rgn);
	DisposeRgn(saveRgn);
}

/* CenterRect
 * Centers Rect *b in Rect *a
 */ 
void CenterRect(Rect *a, Rect *b)
{
	Rect	aTemp = *a, //safety
			bTemp = *b;
	
	b->top = (aTemp.bottom - aTemp.top - (bTemp.bottom - bTemp.top)) / 2 + aTemp.top;
	b->bottom = b->top + (bTemp.bottom - bTemp.top);
	b->left = (aTemp.right - aTemp.left - (bTemp.right - bTemp.left)) / 2 + aTemp.left;
	b->right = b->left + (bTemp.right - bTemp.left);
}

void SetGrey(RGBColor *rgb, short shade)
{
	rgb->red = rgb->green = rgb->blue = shade;
}

// good for copybits...
void SetStdForeNBackColor(void)
{
	/*RGBColor black, white;
	SetGrey(&black, 0x0000);
	SetGrey(&white, 0xFFFF);
	
	RGBForeColor(&black); RGBBackColor(&white);*/
	ForeColor(blackColor); BackColor(whiteColor);
}


/*
 * String Routines 
 */

/* PStringCopy
 * Copies pascal string a into b.
 */
void PStringCopy(Str255 a, Str255 b)
{
	BlockMove(a,b, a[0] + 1);
}

// cat a + b, return in a
// move this to MMR.c
void PStringCat(Str255 a, Str255 b)
{
	short len;
	
	if ((a[0] + b[0]) > 255)
		len = 255-a[0];
	else 
		len = b[0];
		 
	//bcopy((char *)&(b[1]),(char *)&(a[a[0]+1]),len);
	BlockMove(&(b[1]),&(a[a[0]+1]),len);
	a[0] += len;
}

/* Str2Double
 * Given a Str255, it converts it to a double precision number 
 * (to the best of it's ability).
 *
 * Pre: s is a string containing a floating point number
 * Post: the correct double is returned
 */
/* CRAP!! THIS DOES NOT WORK !! I'll have to apply a Kludge to it. MDT */
double Str2Double(Str255 s)
{
	decimal d;
	short ignore;
	short index = 1; /* Pascal string starts at 1 */

	if (s[0] == 0)	
		return 0;		/* s = "\p" */
		
	TagZero(s);  /* KLUDGE */
	str2dec((const char *)s, &index, &d, &ignore);
	if (ignore = false) { SysBeep(3); SysBeep(3); SysBeep(3); }
	return dec2num(&d);
}

/* TagZero
 * Kludge for Str2Ext
 */
void TagZero(Str255 s)
{
	short index= 0, i;
	
	//index = s[0];
	for (i = 1; i<=s[0]; i++) {
		if (s[i] == '.') {
			return;				/* there is a decimal - no worries */
		}
	}
	
	s[s[0]+1] = '.';
	s[s[0]+2] = '0';
	s[0] += 2;
}

/* Double2Str
 * Given a double, it converts it to a Str255, by way of C string 
 * (to the best of it's ability).
 *
 * Pre: e contains the number; s points to a valid string
 * Post: s contains the proper string.
 */
/* Last program that used this didn't work :( (PPC Fractal Generator)
 * But this one seems to! :)
 */
void Ext2Str (Str255 s, double e)
{
	decform df;
	decimal d;
	char	i=0;	// For C String Parsing
	char	*t;		// For C String Parsing

	df.style = FIXEDDECIMAL;
	df.digits = 1;
	num2dec(&df,e,&d);
	dec2str(&df,&d,(char *)&(s[1]));
	
	/*** KLUGE ALERT! ***/
	t = (char *)s;
	//s[0] = (char)d.sig.length + 1; /* Better to use String Length a-la strlen */
	while(*(++t)) i++;	/* This is strlen (for Pascal strings - note the ++t); I'm avoiding the ANSI routines */
	s[0] = i;
}

/*
 * Cursor Routines
 */


/* FixCursor
 * Set the cursor to the Arrow cursor and make it visible, please.
 */
void FixCursor(void)
{
	SetCursor(&qd.arrow);
	ShowCursor();
}

/* SetMyCursor
 * Changes the cursor to the CURS resource w/ ID number id.  Pretty straight forward.
 */
void SetMyCursor(short id)
{
	CursHandle	cursH;
	
	cursH = GetCursor( id );
	HLock ((Handle) cursH);

	SetCursor( *cursH );
	HUnlock((Handle) cursH);
}

/* SetMyCCursor
 * Changes the cursor to the crsr resource w/ ID number id.  Pretty straight forward.
 */
void SetMyCCursor(short id)
{
	CCrsrHandle	cursH;
	
	cursH = GetCCursor( id );
	//HLock ((Handle) cursH);

	SetCCursor( cursH );
	//HUnlock((Handle) cursH);
}

/*
 * Bit Depth Routines
 */

Boolean IsColor(WindowPtr w)
{
	PixMapHandle	pixMap;
	Boolean			isColor;
	
	pixMap = ((CGrafPtr)w)->portPixMap; 
	HLock((Handle)pixMap);
	isColor = ((* pixMap)->pixelSize!=1) ? true : false;
	HUnlock((Handle)pixMap);
	return isColor;
}

/*
 * Event Routines
 */

void YieldEvents (void)
{
	EventRecord e;
	
	(void)EventAvail(everyEvent, &e);
}

Boolean IsOptionKeyDown(void)
{
	EventRecord e;
	
	// determine the option key setting
	EventAvail(mouseDown+mouseUp+keyDown+autoKey+keyUp, &e) ;
	if (e.modifiers & optionKey) {
		return true;
	}
	return false;
}

/*
 * File Routines
 */

Boolean EqualFSSpec(FSSpec *a, FSSpec *b)
{
	return ((a->vRefNum == b->vRefNum) &&
	    (a->parID == b->parID) &&
	    EqualString(a->name,b->name,true,true));
}