/* 
 * Map Window.c
 */
/* Adapted for multiwindow - 2/2/96
 */

#include <QDOffscreen.h>
#include "MMR.h"
#include "Map Window.h"
#include "Maps.h"
#include "Menusx.h"
#include "Scenario.h"
#include "Tiles.h"
#include "ToolPaletteCDEF.h"
#include "Tools.h"
#include "TileMenuCommon.h"
#include "Cursors30.h"
#include "Monsters.h"
#include "TileInfo.h"
#include "WinManager.h"
#include "Alertsx.h"
#include "WindowUtil.h"

#include "Locations.h" // mdt 5/26/96

#define kMapWindowID		129
#define kMapWindowBitDepth	8
#define kMinWindowWidth		256
#define kMinWindowHeight	256
#define kScrollBarOffset	15

#define kHScroll			1
#define kVScroll			2
#define kToolPalette		3

#define kCrsrArrow			0
#define kCrsrWatch			129
#define kCrsrEraser			136
#define kCrsrPencil			137
#define kCrsrEyeDropper		138
#define kCrsrPaintBucket	139
#define kCrsrRectSelect		140
#define kCrsrEyeBeam		141
#define kCrsrEyeGlass		142
#define kCrsrEyeGlass2		143
#define kCrsrNoGood			144

WindowPtr	NewMapWindow(short mapID);
void CloseMapWindow(WindowPtr w);
OSErr ResizeMapWindow(WindowPtr window, EventRecord *e);
void MoveMapWindow(WindowPtr w, EventRecord *e, Rect *limitRect);
void MapUpdate(WindowPtr w);
void DrawMapWindow(WindowPtr w);
void CalculateMapWindow(WindowPtr w);
void SetTileType(WindowPtr w,short tile);
void MWStartCursTracking(WindowPtr w, Rect *r);
void MWStartMonsCursTracking (WindowPtr w,Rect *r);
void HandleMapKey(WindowPtr w,EventRecord *e, short theChar);
void HandleMapContentClick(WindowPtr w, EventRecord *e);
void HandleMapEdit(WindowPtr w, Point mousePt);
void MPtoQDRect(WindowPtr w, Rect *MPRect, Rect *QDRect);
void InvertSelectedRect(WindowPtr w);
OSErr MakeToolPalette(WindowPtr w);
void DisposeToolPalette(WindowPtr w);
Boolean IsTilePopupClick(WindowPtr w, Point localPt);
OSErr MakeScrollBars(WindowPtr w);
void MoveMapScrollBars(WindowPtr w);
pascal void SBActionProc (ControlHandle theControl, short thePart);
void ClipOutScrollBars(WindowPtr w);


WindowPtr	NewMapWindow(short mapID)
{
	WindowPtr	w;
	Rect mapRect;
	OSErr err;
	MapWindowData *mwData;
	// for zooming MDT 3/17/96
	WindowPeek wPeek;
	WStateData *wsdp;
		
	if ((w = FindMapWindow(mapID)) != nil) {
		SelectWindow(w);
		return w;
	}

	w = GetNewCWindow(kMapWindowID,nil, (WindowPtr)-1L);
	
	if (!w) {
		err = ResError();
		goto fail;
	}
	/*MoveWindow(w,2,40,true);
	SizeWindow(w,qd.screenBits.bounds.right-qd.screenBits.bounds.left-4,
				qd.screenBits.bounds.bottom-qd.screenBits.bounds.top-43,false);*/
	
	
	// set Zoom info. MDT 3/17/96
	wPeek = (WindowPeek)w;
	wsdp = (WStateData*) *(wPeek->dataHandle);
	SetStdZoomRect(&wsdp->stdState);
	
	mwData = (MapWindowData *)NewPtr(sizeof(MapWindowData));
	if (!mwData) goto fail;
	
	// initialize map window globals (so that when things fail, we know what to free)
	mwData->mapWorld = nil;
	mwData->mapVScroll = nil;
	mwData->mapHScroll = nil;
	mwData->toolPalette = nil;
	mwData->mapID = mapID;
	mwData->tileType = 0;//water
	mwData->dirty = false;
	mwData->crsrID = kCrsrArrow;
	SetRect(&mwData->selectedPts,-1,-1,-1,-1);

	mwData->mapWindow = w; 	
	CalcMapRect(w,&mapRect);
	// if (err = MakeGWorld(&(mwData->mapWorld),&mapRect,kMapWindowBitDepth));
		// Do nothing. mapWorld will equal nil; that's all we need to know.		
		//goto fail; MDT 3/13/96

	/*{
		DisposeWindow(w);
		return nil;
	}*/
	
	
	SetWRefCon(w, (long)mwData);
	RegisterWindow(w,nil,kOutdoorsMap); // for now - MDT 2/17/96
	SetWTitle(w, gScenario.outsideMaps[mapID].mapName);   
	BuildWindowMenu();
	
	SetPort(w);
	ShowWindow(w);
	if (err = MakeToolPalette(mwData->mapWindow)) goto fail;//DebugStr("\pFailed to make tool palette!");
	MyDrawGrowIcon(w);
	if (err = MakeScrollBars(w)) goto fail;
	/*{
		DebugStr("\pFailed to make Scrollbars!");
		CloseMapWindow(w);
		return nil;
	}*/
	CalculateMapWindow(w);
		
	return w; 
fail:
	if (w) {
		if (mwData) {
			UnregisterWindow(w); //can be called unnecessarilly
			BuildWindowMenu(); 
			//EndCursTracking(w); //can be called unnecessarilly (i.e., before BeginCursTracking)
			CCRemoveWindow(w);
			
			if (mwData->mapVScroll) DisposeControl(mwData->mapVScroll);
			if (mwData->mapHScroll) DisposeControl(mwData->mapHScroll);
			if (mwData->toolPalette)DisposeToolPalette(w);
			if (mwData->mapWorld) DisposeGWorld(mwData->mapWorld);
			DisposePtr((Ptr)mwData);	
		}
		DisposeWindow(w);
	}
// DoError for now.
	DoError("\pUltima III Editor can't open any more Map Windows. Try giving the editor more memory.",err,kNotFatal);

	return nil;
}

void CloseMapWindow(WindowPtr w)
{
	MapWindowData *mwData;
	
	if (!w) return;
	
	UnregisterWindow(w);
	BuildWindowMenu();
	mwData = (MapWindowData *) GetWRefCon(w);
	
	//EndCursTracking(w); // THIS IS VERY VERY IMPORTANT !!!!!
	CCTrackCursor(w,false);
	CCRemoveWindow(w);
	mwData->crsrID = kCrsrArrow; // for luck
	mwData->tileType = 0;
	
	
	if (mwData->mapVScroll) DisposeControl(mwData->mapVScroll);
	if (mwData->mapHScroll) DisposeControl(mwData->mapHScroll);
	if (mwData->toolPalette)DisposeToolPalette(w);
	if (mwData->mapWorld) DisposeGWorld(mwData->mapWorld);
	DisposeWindow(w);
	DisposePtr((Ptr)mwData);	
}

// obsolete: ForEachMapWindowDo
/*void CloseAllMapWindows(void)
{
	long i,count,type;
	WindowPtr w;
	
	count =  CountWindows();
	for (i=0; i<count; i++) {
		w = FindNthWindow(1); // in closing the map window, our list shrinks.
		if (w == nil) continue;
		type = FindWindowType(w);
		if (type == kOutdoorsMap) CloseMapWindow(w);
	}
}*/

void ForEachMapWindowDo(void (*func)(WindowPtr wp))
{
	long i,count,type;
	WindowPtr w;
	
	count =  CountWindows();
	for (i=count; i>0; i--) { // we count down so that we can remove windows with this call if we need to.
		w = FindNthWindow(i); 
		if (w == nil) continue;
		type = FindWindowType(w);
		if (type == kOutdoorsMap) (*func)(w);
	}
}

WindowPtr FindMapWindow(short mapID)
{
	short i,max;
	WindowPtr w;
	MapWindowData *mwData;
	
	max = CountWindows();
	for (i=0; i<max; i++) {
		w = FindNthWindow(i+1);
		mwData = (MapWindowData *) GetWRefCon(w);
		if (mwData->mapID == mapID) return w;
	}
	
	return nil;
}

// 1/22/96 - is the max width right?!?
OSErr ResizeMapWindow(WindowPtr window, EventRecord *e)
{
	Rect sizeRect,eraseRect,tpRect;
	OSErr err = noErr;
	long windSize;
	short h,w;
	RgnHandle rgn,saveRgn;
	MapWindowData *mwData;
	long mpSize;
		
	if (!window) return; if (!e) return;
	mwData = (MapWindowData *) GetWRefCon(window);
	mpSize = gScenario.outsideMapsSize[mwData->mapID].h; // mdt 3/11/96 replace 64s.

	SetPort(window);
	tpRect = (*(mwData->toolPalette))->contrlRect;

	SetRect(&sizeRect, kMinWindowWidth,
		tpRect.bottom+16+36+16+kScrollBarOffset, // min height 1/22/96 
		(32*mpSize+kScrollBarOffset) + (kToolSmallOffset * 2) + kToolWidth, // make room for tools
		(32*mpSize+kScrollBarOffset));

	windSize = GrowWindow(mwData->mapWindow, e->where, &sizeRect);
	
	if (windSize) {	
		sizeRect = window->portRect;
		sizeRect.bottom -= 14; sizeRect.right -= 14;

		h = HiWord(windSize);
		w = LoWord(windSize);
		SizeWindow(window, w, h, true);
		eraseRect = (window)->portRect;
		
		// Clip out old screen region
		saveRgn = NewRgn();
		GetClip(saveRgn);
		rgn = NewRgn();
		OpenRgn();
		FrameRect(&eraseRect);
		FrameRect(&sizeRect);
		CloseRgn(rgn);
		SetClip(rgn);
		DisposeRgn(rgn);
		
		// Erase new screen region
		EraseRect(&eraseRect);
	
		SetClip(saveRgn);
		DisposeRgn(saveRgn);
		
		InvalRect(&window->portRect); // we force an update of the window 

		MoveMapScrollBars(window);
		MyDrawGrowIcon(window);

		if (mwData->mapWorld) DisposeGWorld(mwData->mapWorld); // MDT 3/13/96
		CalcMapRect(mwData->mapWindow,&sizeRect);
		//err = MakeGWorld(&mwData->mapWorld,&sizeRect,kMapWindowBitDepth);
		// do nothing on err MDT 3/13/96
		CalculateMapWindow(window);
		
		if (mwData->crsrID != kCrsrArrow) {
			//EndCursTracking(mwData->mapWindow);
			CCRemoveWindow(mwData->mapWindow);
			if (mwData->crsrID == kCrsrEyeGlass) {
				MWStartMonsCursTracking(mwData->mapWindow,&sizeRect);
			} else {
				MWStartCursTracking(mwData->mapWindow,&sizeRect);
			}
		}
	}
	
	return err;
}

/* AutoSizeMapWindow
 * To be called when a Map Window is automatically sized by some external source. 
 * i.e., zooming, stacking, tiling, etc.
 */
void AutoSizeMapWindow(WindowPtr w)
{
	MapWindowData *mwData;
	Rect mapRect;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	MoveMapScrollBars(w);
	MyDrawGrowIcon(w);
		
	if (mwData->crsrID != kCrsrArrow) {
		CalcMapRect(w,&mapRect);
		//EndCursTracking(w);
		CCRemoveWindow(w);
		if (mwData->crsrID == kCrsrEyeGlass) {
			MWStartMonsCursTracking(w,&mapRect);
		} else {
			MWStartCursTracking(w,&mapRect);
		}
	}	
}

void MoveMapWindow(WindowPtr w, EventRecord *e, Rect *limitRect)
{
	Rect mapRect;
	MapWindowData *mwData;
	
	if (!w) return; if (!e) return; if (!limitRect) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	DragWindow(w, e->where, limitRect);

	if (mwData->crsrID != kCrsrArrow) {
		CalcMapRect(w,&mapRect);
		//EndCursTracking(w); unneeded MDT 3/27/96
		if (mwData->crsrID == kCrsrEyeGlass) {
			MWStartMonsCursTracking(w,&mapRect);
		} else {
			MWStartCursTracking(w,&mapRect);
		}
	}	
}

void MapActivate(WindowPtr w, Boolean activate)
{
	MapWindowData *mwData;
	Rect r;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	r = w->portRect;
	r.left = r.right - 14;
	r.top = r.bottom - 14;

	if (activate) {
		HiliteControl(mwData->mapHScroll,0);
		HiliteControl(mwData->mapVScroll,0);
		HiliteControl(mwData->toolPalette,0);
		MyDrawGrowIcon(w);
	} else {
		HiliteControl(mwData->mapHScroll,255);
		HiliteControl(mwData->mapVScroll,255);
		HiliteControl(mwData->toolPalette,255);
		SetPort(w);
		EraseRect(&r);
	}
}


void MapUpdate(WindowPtr w)
{
	BeginUpdate(w);
	MyDrawGrowIcon(w);
	DrawControls(w);
	DrawMapWindow(w);
	EndUpdate(w);
}

void DrawMapWindow(WindowPtr w)
{
	RgnHandle saveRgn;
	Rect		mapRect,r;
	
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	SetPort(w);

	if ((mwData->dirty)) {
		CalculateMapWindow(w);
		mwData->dirty = false;
	}

	saveRgn = NewRgn();
	GetClip(saveRgn);

	SetStdForeNBackColor();
	ClipOutScrollBars(w);
	
	// draw a line
	CalcMapRect(w,&mapRect);
	MoveTo(mapRect.left-1, mapRect.top);
	LineTo(mapRect.left-1, mapRect.bottom);
	
	// draw map
	if (mwData->mapWorld) { // copy bits if the GWorld exists. MDT+8 3/13/96
		CopyBits(&((GrafPtr)(mwData->mapWorld))->portBits, &((GrafPtr)w)->portBits,
		 &(mwData->mapWorld)->portRect, &mapRect,  srcCopy, 0);
	} else { // else draw directly into the window. 
		short x,y;
		GWorldPtr world;
		OSErr err;
		
		x = GetCtlValue(mwData->mapHScroll);
		y = GetCtlValue(mwData->mapVScroll);

		err = MakeGWorld(&world, &mapRect, 8);
		if (err == noErr) {
			DrawMap(world, &mapRect, mwData->mapID, x,y);
			CopyBits(&((GrafPtr)(world))->portBits, &((GrafPtr)w)->portBits,
			 &(world)->portRect, &mapRect,  srcCopy, 0);
			DisposeGWorld(world);
		} else {
			ClipRect(&mapRect);
			DrawMap((GWorldPtr)w, &mapRect, mwData->mapID, x,y);
		}
	}

	DrawTilePopup(w);
	InvertSelectedRect(w);
	
	SetClip(saveRgn);
	DisposeRgn(saveRgn);
	
	//SetProperMouseCursor(w); // fix for pokey window drawing.
}

void CalculateMapWindow(WindowPtr w)
{
	MapWindowData *mwData;
	short x,y;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	if (!(mwData->mapWorld)) return; // bail if no GWorld. MDT 3/13/96
	
	x = GetCtlValue(mwData->mapHScroll);
	y = GetCtlValue(mwData->mapVScroll);
	DrawMap(mwData->mapWorld, &mwData->mapWorld->portRect, mwData->mapID, x,y);
}

void CalcAndDrawMapWindow(WindowPtr w) // called from events.c
{
	SetPort(w);
	CalculateMapWindow(w);
	DrawMapWindow(w);
}

/* SetMapID - Obsolete w/ multi-window code.
void SetMapID(short mapID)
{
	gMWGlobals.mapID = mapID;
}*/

void SetTileType(WindowPtr w,short tile)
{
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	mwData->tileType = tile;
}

void MWStartCursTracking(WindowPtr w, Rect *r)
{
	RgnHandle mouseRgn;
	Rect rect = *r;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	if (mwData->crsrID != kCrsrArrow) {
		LocalToGlobal(&topLeft(rect));
		LocalToGlobal(&botRight(rect));
		mouseRgn = NewRgn();
		SetPort(w); // for OpenRgn
		OpenRgn();
		FrameRect(&rect);
		CloseRgn(mouseRgn);
		
		CCRemoveWindow(w);
		CCSetRegion(w,mouseRgn,CCNewCursor(single,mwData->crsrID,true),
			nil,
			CCNewCursor(single,kCrsrEyeDropper,true),
			nil, nil);
		CCTrackCursor(w,true);
	}
}

void MWStartMonsCursTracking (WindowPtr w,Rect *r)
{
	RgnHandle monsRgn, windRgn;
	Rect mapRect = *r,monsRect,tmpRect,emptyRect;
	short i;
	MonsterRec monster;
	MapWindowData *mwData;
	Point	pos;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	SetRect(&emptyRect,0,0,0,0);
	monsRgn = NewRgn(); windRgn = NewRgn();

	// set mapRect to global coordinates.
	LocalToGlobal(&topLeft(mapRect)); LocalToGlobal(&botRight(mapRect));
	
	// build our monster Rgn:
	SetPort(w);
	OpenRgn();
	
	//   loop through all the monsters for our map
	for (i=0; i<kMaxMonsters; i++) {
		FindMonsterByNumber(mwData->mapID, i, &monster);
		if (monster.monsTileType == 0) continue ; // no monster
		
		// set monster's position
		monsRect.left = monsRect.right = monster.monsPosition.h;
		monsRect.top = monsRect.bottom = monster.monsPosition.v;
		MPtoQDRect(w,&monsRect,&tmpRect);
		LocalToGlobal(&topLeft(tmpRect)); LocalToGlobal(&botRight(tmpRect));
		SectRect(&tmpRect, &mapRect, &monsRect); // clip to map area; store in monsRect
		if (EqualRect(&monsRect,&emptyRect)) continue ; // not visible
		
		FrameRect(&monsRect);
	}

	if (mwData->mapID == 12) { // 12 = Sosaria
	//   loop through all the cities/dungeons for our map
		for (i=0; i<19; i++) {
			FindLocationByID(kLocCity,&pos,i); // cities also returns dungeons - for now
			
			// set position
			monsRect.left = monsRect.right = pos.h;
			monsRect.top = monsRect.bottom = pos.v;
			MPtoQDRect(w,&monsRect,&tmpRect);
			LocalToGlobal(&topLeft(tmpRect)); LocalToGlobal(&botRight(tmpRect));
			SectRect(&tmpRect, &mapRect, &monsRect); // clip to map area; store in monsRect
			if (EqualRect(&monsRect,&emptyRect)) continue ; // not visible
			
			FrameRect(&monsRect);
		}
		// loop through moongates
		for (i=0; i<8; i++) {
			FindLocationByID(kLocMoongate,&pos,i); // cities also returns dungeons - for now
			
			// set position
			monsRect.left = monsRect.right = pos.h;
			monsRect.top = monsRect.bottom = pos.v;
			MPtoQDRect(w,&monsRect,&tmpRect);
			LocalToGlobal(&topLeft(tmpRect)); LocalToGlobal(&botRight(tmpRect));
			SectRect(&tmpRect, &mapRect, &monsRect); // clip to map area; store in monsRect
			if (EqualRect(&monsRect,&emptyRect)) continue ; // not visible
			
			FrameRect(&monsRect);
		}
	}

	CloseRgn(monsRgn);
	
	// build our window region - see MWStartCursTracking
	OpenRgn();
	FrameRect(&mapRect); // start with mapRect,
	FrameRgn(monsRgn);	 // and exclude the monsRgn (just to make sure we really understand overlap)
	CloseRgn(windRgn);
	
	// Now set our cursor information
	CCRemoveWindow(w); // remove the window (to purge all previously existing regions
	CCSetRegion(w,monsRgn,CCNewCursor(single,kCrsrEyeGlass2,true),
		nil,
		CCNewCursor(single,kCrsrEyeDropper,true),
		nil, nil); // set the monster region with the special cursor
	CCSetRegion(w,windRgn,CCNewCursor(single,mwData->crsrID,true),
		nil,
		CCNewCursor(single,kCrsrEyeDropper,true),
		nil, nil); // set the ambient region
	CCTrackCursor(w,true); // enable cursor tracking for our window

}

void HandleMapKey(WindowPtr w,EventRecord *e, short theChar)
{
	short letter = -1;
	MapWindowData *mwData;
	
	if (!w) return; if (!e) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	
	if (mwData->selectedPts.left == -1) return; // if no cursor.
	
	if ((theChar >= 'a') && (theChar <= 'z')) theChar -= 'a' - 'A'; // 'a' - ('a' - 'A') = 'a' - 'a' + 'A' or 'A'  :-)
	
	if (theChar == ' ') letter = 0;
	if ((theChar >= 'A') && (theChar <= 'Z')) letter = theChar - 'A' + 1;
	if (letter != -1) {
		DoLetterKeyPress(letter, true, mwData);	
		return;
	}
	
	// handle control characters
	/*switch (theChar) {
	case 0xFF: // delete
	case 0x08: // backspace
		gMWGlobals.cursor.h--;
		letter = -2; // erase mode
		break;
	case 28:   // left arrow
		gMWGlobals.cursor.h--;
		letter = -1; // draw nothing
		break;
	case 29: // right arrow
		gMWGlobals.cursor.h++;
		letter = -1; // draw nothing
		break;
	case 30:   // up arrow
		gMWGlobals.cursor.v--;
		letter = -1; // draw nothing
		break;
	case 31: // down arrow
		gMWGlobals.cursor.v++;
		letter = -1; // draw nothing
		break;
	default:
		SysBeep(3);
		return;
	}
	if (gMWGlobals.cursor.h < 0) {
		gMWGlobals.cursor.h = 63;
		gMWGlobals.cursor.v--;
	}
	if (gMWGlobals.cursor.h > 63) {
		gMWGlobals.cursor.h = 0;
		gMWGlobals.cursor.v++;
	}
	if (gMWGlobals.cursor.v < 0) {
		gMWGlobals.cursor.v = gMWGlobals.cursor.h = 0;
	}
	if (gMWGlobals.cursor.v > 63) {
		gMWGlobals.cursor.v = gMWGlobals.cursor.h = 0;
	}
	DoLetterKeyPress(letter,false, &gMWGlobals);	*/
		
}


void HandleMapContentClick(WindowPtr w, EventRecord *e)
{
	short 			thePart,oldVal,val;
	ControlHandle	theControl;
	Point			thePoint;
	ControlActionUPP controlAction;
	Rect r;
	RgnHandle		mouseRgn;
	MapWindowData   *mwData;
	OSErr 			err = noErr;
	
	if (!w) return; if (!e) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	
	SetPort(w);
	SetStdForeNBackColor();
	thePoint = e->where;
	GlobalToLocal(&thePoint);
	
	// first check to see if we clicked in the edit map.
	CalcMapRect(w,&r);
	if (PtInRect(thePoint,&r)) {
		gScenario.scenDirty = true; // something will probably change.
		HandleMapEdit(w,thePoint); // draw.... or whatever
		return;
	}
	
	// if not there, check our popup.
	if (IsTilePopupClick(w,thePoint)) {
		MenuHandle menu;
		long menuChoice;
		
		SetRect(&r,0,0,34,34);
		OffsetRect(&r,17,(*mwData->toolPalette)->contrlRect.bottom + 17);
		TMInvertItem2(&r);
		
		menu = GetMenu(200);
		InsertMenu(menu,-1);
		//SetPt(&thePoint, 17, 206);
		SetPt(&thePoint,14, (*mwData->toolPalette)->contrlRect.bottom + 14);
		LocalToGlobal(&thePoint);
		menuChoice = PopUpMenuSelect(menu, thePoint.v, thePoint.h, 1);
		DeleteMenu(200);
		TMInvertItem2(&r);

		if (HiWord(menuChoice) == mTiles) {
			menuChoice = (long)LoWord(menuChoice);
			SetTileType(w,((short)menuChoice-1) * 4);
			CheckTMItem((short)menuChoice);
			DrawTilePopup(w);
		}
		return ;
	}
	
	// If not there, check our controls.
	thePart = FindControl(thePoint, w, &theControl);
	if (theControl){
		switch ((*theControl)->contrlRfCon) {
		case kVScroll:
		case kHScroll:
			// make a GWorld scratchspace for us, so we aren't constantly making and
			// disposing one while tracking. *MDT(-1,+1) 3/17/96
			err = MakeGWorld(&mwData->mapWorld,&r,8);
			if (thePart == inThumb) {
				controlAction = NewControlActionProc(nil);
				thePart = TrackControl(theControl, thePoint, controlAction);
				DisposeRoutineDescriptor(controlAction);
				CalculateMapWindow(w);
				DrawMapWindow(w);
			} else { 
				controlAction = NewControlActionProc(SBActionProc);
				thePart = TrackControl(theControl, thePoint, controlAction);
				DisposeRoutineDescriptor(controlAction);
			}
			if (mwData->mapWorld) { //now clean up. MDT+3 3/17/96
				DisposeGWorld(mwData->mapWorld); 
				mwData->mapWorld = nil;
			}
			
			if (mwData->crsrID == kCrsrEyeGlass) {
				MWStartMonsCursTracking(w,&r);
			}
			break;
		case kToolPalette:
			oldVal = GetCtlValue(mwData->toolPalette);
			thePart = TrackControl(mwData->toolPalette,e->where, (ControlActionUPP) -1L);
			val = GetCtlValue(mwData->toolPalette);
			if (val < 1) {
				SetCtlValue(mwData->toolPalette,oldVal);
				val = oldVal;
			}
			
			// reset our selected rect...
			if (((val != kToolSelect) && (val != kToolPaintBucket)) ||
				 (oldVal == kToolLetter))
			 {
				if (mwData->selectedPts.left!=-1) {
					InvertSelectedRect(w);
					SetRect(&mwData->selectedPts,-1,-1,-1,-1);
				}
			}
			
			// set mouse pointer
			CalcMapRect(w,&r);
			switch (val) {
			case kToolPointer:
				mwData->crsrID = kCrsrArrow;
				//EndCursTracking(w);
				CCRemoveWindow(w);
				break;
			case kToolSelect:
				mwData->crsrID = kCrsrRectSelect;
				MWStartCursTracking(w,&r);
				break;
			case kToolEraser:
				mwData->crsrID = kCrsrEraser;
				MWStartCursTracking(w,&r);
				break;
			case kToolPencil:
				mwData->crsrID = kCrsrPencil;
				MWStartCursTracking(w,&r);
				break;
			case kToolEyeDropper:
				mwData->crsrID = kCrsrEyeDropper;
				MWStartCursTracking(w,&r);
				break;
			case kToolPaintBucket:
				mwData->crsrID = kCrsrPaintBucket;
				MWStartCursTracking(w,&r);
				break;
			case kToolLetter:
				mwData->crsrID = kCrsrEyeBeam;
				MWStartCursTracking(w,&r);
				break;
			case kToolMonsterInfo:
				mwData->crsrID = kCrsrEyeGlass;
				MWStartMonsCursTracking(w,&r);
				break;
			default:
				mwData->crsrID = kCrsrNoGood;
				MWStartCursTracking(w,&r);
			}
			break;
		default:
			;
		} // switch
	}
}

// mousePt is in local coordinates. 
void HandleMapEdit(WindowPtr w, Point mousePt)
{
	Rect mapRect;
	short scrollDelta, activeTool;
	short scrollHMax,scrollHVal,scrollVMax,scrollVVal;
	Point 	mapPt, // mapPt is synonymous with the coordinates in our map array. It's the fundamental unit of tiles.
			saveMapPt,
			fillPt, // passed into fill...
			initialMapPt; // used for scaling area selections
	Boolean dirty = false,monsKluge = false;
	Boolean dragKluge = false;
	RgnHandle		save,clip;
	MapWindowData *mwData;
	long mpSize;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	mpSize = gScenario.outsideMapsSize[mwData->mapID].h; // mdt 3/11/96 replace 64s.
	
	//lock down maps. 
	HLock(gScenario.outsideMaps[mwData->mapID].map);
	HLock(gScenario.outsideMaps[mwData->mapID].mapCopy);
	
	SetPt(&saveMapPt,-1,-1); // initial values.
	activeTool = GetCtlValue(mwData->toolPalette);
	if (IsOptionKeyDown()) activeTool = kToolEyeDropper; // for now ... 
	
	CalcMapRect(w, &mapRect);	// get the mapRect
	
	scrollHMax=GetCtlMax(mwData->mapHScroll);
	scrollVMax=GetCtlMax(mwData->mapVScroll);

	// note initialMapPt will always be within the mapRect.
	initialMapPt.h = (GetCtlValue(mwData->mapHScroll) + (mousePt.h - mapRect.left)) / 32;
	initialMapPt.v = (GetCtlValue(mwData->mapVScroll) + (mousePt.v - mapRect.top)) / 32;
	
	// set the clipping area
	save = NewRgn();
	clip = NewRgn();
	GetClip(save);
	RectRgn(clip,&mapRect);
	SetClip(clip);
	
	// kludge for dragging things //monsters
	if (activeTool == kToolPointer) {
		FindMonsterByPosition(mwData->mapID, initialMapPt, &mwData->monster);
	}
	
	// TIME CRITICAL LOOP:
	do { // do while mouse down:
		
		// if our point is out of the mapRect, adjust the scroll bars; this has the 
		// effect of auto-scrolling our window.
		// Remember, the ControlManager handles going below min and above max for us.
		if (!PtInRect(mousePt,&mapRect)) {
			scrollDelta = (IsOptionKeyDown()) ? 1 : 32;
			scrollHVal = GetCtlValue(mwData->mapHScroll);
			scrollVVal = GetCtlValue(mwData->mapVScroll);
			
			if ((mousePt.h < mapRect.left) && (scrollHVal > 0)) {
				SetCtlValue(mwData->mapHScroll, scrollHVal -scrollDelta);
				dirty = true;
			} else if ((mousePt.h > mapRect.right) && (scrollHVal < scrollHMax)) {
				SetCtlValue(mwData->mapHScroll, scrollHVal +scrollDelta);
				dirty = true;
			}
			
			if ((mousePt.v < mapRect.top) && (scrollVVal > 0)) {
				dirty = true;
				SetCtlValue(mwData->mapVScroll, scrollVVal -scrollDelta);
			} else if ((mousePt.v > mapRect.bottom) && (scrollVVal < scrollVMax)) {
				SetCtlValue(mwData->mapVScroll, scrollVVal+scrollDelta);
				dirty = true;
			}
			// can we make this faster?
			if (dirty) {
				CalcAndDrawMapWindow(w); // only if we NEED to.
				SetClip(save);
				Draw1Control(mwData->mapHScroll); Draw1Control(mwData->mapVScroll);
				SetClip(clip);
				dirty = false;
				dragKluge = true;
			}
		}
		
		// determine if we're over the same tile. If we've moved out of the tile we
		// were over, go ahead and invoke a tool. We don't want to draw a specific 
		// tile every time we go through the loop, we only want to draw when we first
		// move over it.  
		// HINT: We will always go through this code if we've scrolled above.
		
		// mdt test: move mousePt so it's in mapRect.
		// yes I like that much better!!
		if (mousePt.h < mapRect.left) mousePt.h = mapRect.left;
		if (mousePt.h > mapRect.right) mousePt.h = mapRect.right-1; // -1 is mapPt kludge.
		if (mousePt.v < mapRect.top) mousePt.v = mapRect.top;
		if (mousePt.v > mapRect.bottom) mousePt.v = mapRect.bottom-1;// -1 is mapPt kludge.
		
		mapPt.h = (GetCtlValue((mwData->mapHScroll)) + (mousePt.h - mapRect.left)) / 32;
		mapPt.v = (GetCtlValue((mwData->mapVScroll)) + (mousePt.v - mapRect.top)) / 32;

		// kluge
		if (dragKluge) {
			
			if ((activeTool == kToolPointer) && (mwData->monster.monsTileType != -1)) {
				mwData->dirty = DoDragMonster(initialMapPt,mapPt,mwData);
				//initialMapPt = mapPt;
			}
			dragKluge = false;
		}
		
		// we need to do the drawing tools now ... we need to do the selection tools AFTER
		// we update scrolling.		
		if (!EqualPt(mapPt,saveMapPt)) {
			saveMapPt = mapPt;
			
			switch (activeTool) {
			case kToolPointer:
				if (mwData->monster.monsTileType != -1) {
					mwData->dirty = DoDragMonster(initialMapPt,mapPt,mwData);
					initialMapPt = mapPt;
				}
				break;
			case kToolSelect:
				mwData->dirty = DoSelect(initialMapPt,mapPt, mwData);
				break;
			case kToolEraser:
				mwData->dirty = DoEraser(initialMapPt,mapPt, mwData);
				break;
			case kToolPencil:
				mwData->dirty = DoPencil(initialMapPt, mapPt, mwData);
				break;
			case kToolEyeDropper:
				mwData->dirty = DoEyeDropper(initialMapPt,mapPt, mwData);
				break;
			case kToolPaintBucket:
				fillPt.h = (short)((unsigned char *)(*gScenario.outsideMaps[mwData->mapID].map))[(mpSize * initialMapPt.v)+initialMapPt.h];
				fillPt.v = PtInSelectedRect(w,initialMapPt);
				mwData->dirty = DoPaintBucket(fillPt, mapPt, mwData);
				InvalRect(&w->portRect); // redraw on the update.
				break;
			case kToolLetter:
				mwData->dirty = DoLetter(initialMapPt,mapPt, mwData);
				break;
			case kToolMonsterInfo:
				//SuspendCursTracking();
				CCTrackCursor(w,false);
				DoTileInfoDLOG(mwData->mapID, initialMapPt);
				CalcAndDrawMapWindow(w);
				//ResumeCursTracking(w);
				CCTrackCursor(w,true);
				MWStartMonsCursTracking(w,&mapRect);
				break;
			default:
				;
			}
		}
		
		// calculate new mouse position and do it all over again
		GetMouse(&mousePt);
	} while (Button());
	// END OF TIME CRITICAL LOOP!!
	
	//kludge for dragging monsters
	if ((activeTool == kToolPointer) && (mwData->monster.monsTileType != -1)) {
		mwData->dirty = DoPlotMonster(initialMapPt, mapPt, mwData);
	}
	
	// if our newly selected area is only 1x1, we don't bother with it.
	if ((mwData->selectedPts.left == mwData->selectedPts.right) && 
		(mwData->selectedPts.top == mwData->selectedPts.bottom) &&
		(activeTool == kToolSelect)) {
		InvertSelectedRect(w);
		SetRect(&(mwData->selectedPts),-1,-1,-1,-1);
	}

	
	SetClip(save);
	DisposeRgn(save);
	DisposeRgn(clip);
	HUnlock(gScenario.outsideMaps[mwData->mapID].map);
	HUnlock(gScenario.outsideMaps[mwData->mapID].mapCopy);
}


void MPtoQDRect(WindowPtr w, Rect *MPRect, Rect *QDRect)
{
	MapWindowData *mwData;
	
	if (!w) return; if (!MPRect) return; if (!QDRect) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	
	QDRect->top = (MPRect->top * 32) - GetCtlValue(mwData->mapVScroll);
	QDRect->left = (MPRect->left * 32) - GetCtlValue(mwData->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	QDRect->bottom = QDRect->top + ((MPRect->bottom - MPRect->top + 1) * 32);
	QDRect->right = QDRect->left + ((MPRect->right - MPRect->left + 1) * 32);
}

void InvertSelectedRect(WindowPtr w)
{
	Rect r,mapRect;
	Pattern gray;
	RgnHandle save,clip;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
		
	if (mwData->selectedPts.left==-1) return; // not selected

	save=NewRgn(); clip=NewRgn();
	if ((save == nil) || (clip == nil)) DebugStr("\pnil rgn");
	GetClip(save);
	CalcMapRect(w,&mapRect);
	RectRgn(clip,&mapRect);
	SetClip(clip);
	
	MPtoQDRect(w,&mwData->selectedPts,&r);

	gray.pat[0] = gray.pat[2] = gray.pat[4] = gray.pat[6] = 0xAA; // 10101010
	gray.pat[1] = gray.pat[3] = gray.pat[5] = gray.pat[7] = 0x55; // 01010101

	PenMode(patXor);//This makes it easy to fix the pixels
 	PenPat(&gray);

	FrameRect(&r);

	PenMode(patCopy);
	PenPat(&qd.black);
	SetClip(save);
	DisposeRgn(save);DisposeRgn(clip);
}

Boolean PtInSelectedRect(WindowPtr w,Point p)
{
	Rect r;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	r = mwData->selectedPts;
	if (r.left == -1) return false; // r.left == -1 is an optomization for no selection.
	
	r.right += 1; r.bottom += 1;
	return PtInRect(p,&r);
}

#define kToolControlWidth 64

void CalcMapRect(WindowPtr w,Rect *r)
{
	Rect rect = w->portRect;
	rect.left += (kToolSmallOffset * 2) + kToolWidth; // mask out tool control area
	rect.right -= kScrollBarOffset; rect.bottom -= kScrollBarOffset; // mask out scrollbars
	
	*r = rect;
}


OSErr MakeToolPalette(WindowPtr w)
{
	Rect r;
	ControlDataRec *data;
	PicHandle toolPict;
	OSErr err;
	short i;
	short debug;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	// create data rec
	data = (ControlDataRec*) NewPtr(sizeof(ControlDataRec));
	if (!data) {
		DebugStr("\pMakeToolPalette Errror");
		return MemError();
	}
	
	// set data.tools
	toolPict = GetPicture(kToolPicture);
	if (!toolPict) {
		DebugStr("\pMakeToolPalette Errror");
		return ResError();
	}
		
	r = (*toolPict)->picFrame;
	OffsetRect(&r,-r.left, -r.top); // zero left and top edges of rect.
	err = MakeGWorld(&(data->tools), &r, 8);
	if (err) {
		DebugStr("\pMakeToolPalette Errror");
		return err;
	}
	
	DrawMyPict(data->tools,toolPict); 
	ReleaseResource((Handle)toolPict); // clean up after ourselves.

	// set data.enabled
	data->enabled = (Boolean *)NewPtr(sizeof(Boolean) * kNumTools);
	if (!data->enabled) {
		DebugStr("\pMakeToolPalette Error");
		return MemError();
	}
	for (i=0; i<kNumTools; i++) {	// enable everyone
		(data->enabled)[i] = true;
	}
	
	//set data.numTools
	data->numTools = kNumTools;
	
	// set widths and stuff
	data->borderWidth = data->borderHeight = kControlBorder;
	data->itemWidth = kCItemWidth;
	data->itemHeight = kCItemHeight;
	data->columns = kControlWidth;
	data->rows = (kNumTools + (kControlWidth -1)) / kControlWidth; // or 8 in our example.
	data->orientation = kOrientVertical; // unused.
	
	// set control Rect
	SetRect(&r, kToolSmallOffset,kToolLargeOffset,0,0);
	
	mwData->toolPalette = NewControl(w, &r, "\pTool Palette", true, 1, -1, kNumTools,(128<<4), (long) data);   
	if (!mwData->toolPalette) {
		DebugStr("\pMakeToolPalette Errror");	
		return MemError();
	}

	(*(mwData->toolPalette))->contrlRfCon = kToolPalette; // set RfCon to control id...
	
	return noErr;
}

void DisposeToolPalette(WindowPtr w)
{
	MapWindowData *mwData;
	ControlDataRec *data;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	
	if (!(mwData->toolPalette)) return; // no handle
	
	HLock((Handle)(mwData->toolPalette));
	
	data = (ControlDataRec *)((*(mwData->toolPalette))->contrlData);
	DisposeGWorld(data->tools);
	DisposePtr((Ptr)data->enabled);
	DisposePtr((Ptr)data);
	(*(mwData->toolPalette))->contrlData = (Handle)nil;

	HUnlock((Handle)mwData->toolPalette);
	
	DisposeControl(mwData->toolPalette);
}

void DrawTilePopup(WindowPtr w)
{
	RgnHandle clip, save;
	Rect r,tpRect;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	
	tpRect = (*mwData->toolPalette)->contrlRect;
	
	save = NewRgn();
	clip= NewRgn();
	if ((save == nil) || (clip == nil)) DebugStr("\pnil rgn");
	GetClip(save);

	SetRect(&r,0,0,36,36);
	OffsetRect(&r,16,tpRect.bottom + 16);
	RectRgn(clip,&r);
	SetClip(clip);
	FrameRect(&r);
	InsetRect(&r,2,2);
	PlotTile(gScenario.tiles,(unsigned char)(mwData->tileType),nil,&r);
	SetClip(save);
	DisposeRgn(clip);
	DisposeRgn(save);
}

Boolean IsTilePopupClick(WindowPtr w, Point localPt)
{
	Rect r,tpRect;
	MapWindowData *mwData;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);

	tpRect = (*mwData->toolPalette)->contrlRect;

	SetRect(&r,0,0,36,36);
	//OffsetRect(&r,16,170);
	OffsetRect(&r,16,tpRect.bottom + 16);
	
	return (PtInRect(localPt, &r));
}

// here on out lies scrollbars


//2048 is compiled constant - width of map in pixels;
OSErr MakeScrollBars(WindowPtr w)
{
	MapWindowData *mwData;
	Rect r,mapRect;
	short windWidth,windHeight, // windW&H obsolete?
			mapWidth, mapHeight;
	long mpSize,mpPixSize;
		
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	mpSize = gScenario.outsideMapsSize[mwData->mapID].h; // mdt 3/11/96 replace 64s.
	mpPixSize = mpSize * 32; // aka. 2048

	//set mapWidth & Height
	CalcMapRect(w,&mapRect);
	mapWidth = mapRect.right - mapRect.left;
	mapHeight = mapRect.bottom - mapRect.top;
	
	r = w->portRect;
	windHeight = r.bottom-15;	// these are used to calc. Max. The
	windWidth = r.right-15;		// -15 accounts for both scroll bars.
	
	SetRect( &r,  r.right-15,r.top-1,  r.right+1,r.bottom-14 );
	mwData->mapVScroll = NewControl(w, &r, "\p", true, 0, 0, mpPixSize-mapHeight , scrollBarProc, 0);
	if (!(mwData->mapVScroll)) return MemError();
	(* (mwData->mapVScroll))->contrlRfCon = kVScroll;
	
	if (GetCtlMax(mwData->mapVScroll) > 0) {
		HiliteControl(mwData->mapVScroll, 0);
	} else {
		HiliteControl(mwData->mapVScroll, 255);
	}

	r = w->portRect;
	SetRect( &r,  r.left-1,r.bottom-15,  r.right-14,r.bottom+1 );
	mwData->mapHScroll = NewControl(w, &r, "\p", true, 0, 0, mpPixSize-mapWidth , scrollBarProc, 0);
	if (!mwData->mapHScroll) return MemError();
	(* (mwData->mapHScroll))->contrlRfCon = kHScroll;
	
	if (GetCtlMax(mwData->mapHScroll) > 0) {
		HiliteControl(mwData->mapHScroll, 0);
	} else {
		HiliteControl(mwData->mapHScroll, 255);
	}
	return noErr;
}

void MoveMapScrollBars(WindowPtr w)
{
	short height, width, /*winWidth, winHeight*/mapWidth, mapHeight;
	Rect mapRect;
	MapWindowData *mwData;
	long mpSize, mpPixSize;
	
	if (!w) return;
	mwData = (MapWindowData *) GetWRefCon(w);
	mpSize = gScenario.outsideMapsSize[mwData->mapID].h; // mdt 3/11/96 replace 64s.
	mpPixSize = mpSize * 32; // aka. 2048
	
	width = 16;	// kScrollBarWidth

	CalcMapRect(w, &mapRect);
	mapWidth = mapRect.right - mapRect.left;
	mapHeight = mapRect.bottom - mapRect.top;
	
	MoveControl((mwData->mapVScroll), w->portRect.right-15, w->portRect.top-1);   // IM I pg 325 
	height = w->portRect.bottom - w->portRect.top - 13;
	SizeControl((mwData->mapVScroll), width, height);   // IM I pg 326 
	SetCtlMax((mwData->mapVScroll),mpPixSize - mapHeight);

	MoveControl((mwData->mapHScroll), w->portRect.left-1, w->portRect.bottom-15);   /* IM I pg 325 */
	height = w->portRect.right - w->portRect.left - 13;
	SizeControl((mwData->mapHScroll), height, width);   /* IM I pg 326 */
	SetCtlMax((mwData->mapHScroll),mpPixSize - mapWidth);
}

pascal void SBActionProc (ControlHandle theControl, short thePart)
{
	short controlValue,width,mapWidth,mapHeight,rfCon,upDownButtonDelta = 32;
	EventRecord e;
	Rect mapRect;
	WindowPtr w = (*theControl)->contrlOwner;
	
	CalcMapRect(w,&mapRect);
	mapWidth = mapRect.right - mapRect.left;
	mapHeight = mapRect.bottom - mapRect.top;
	
	controlValue = GetCtlValue(theControl);
	
	if ((* theControl)->contrlRfCon == kHScroll) {
		width = mapWidth; // width of map area minus one tile
	} else {
		width = mapHeight;
	}
	
	// determine the option key setting
	EventAvail(mouseDown+mouseUp+keyDown+autoKey+keyUp, &e) ;
	if (e.modifiers & optionKey) {
		upDownButtonDelta = 1;
	}
	
	switch(thePart){
	case 0:				/* Button released outside of control */
		break;
	case inUpButton:
		controlValue -= upDownButtonDelta ;
		break;
	case inDownButton:	
		controlValue += upDownButtonDelta;
		break;
	case inPageUp:
		controlValue = controlValue - width;
		//controlValue -=32 ;
		break;
	case inPageDown:
		controlValue = controlValue + width ;
		//controlValue += 32;
		break;
	case inThumb:
	/*CalculateMapWindow(w);
	DrawMapWindow(w);*/
	
		break;
	default:			/* Just in case */
		;
	}
	SetCtlValue(theControl, controlValue);
	
	CalculateMapWindow(w);
	DrawMapWindow(w);
}

void ClipOutScrollBars(WindowPtr w)
{
	Rect scrollBar;
	RgnHandle rgn;
	
	rgn = NewRgn();
	if (!rgn) {
		DebugStr("\pClipOutScrollBars");
		ExitToShell();
	}

	SetPort(w); // for OpenRgn
	OpenRgn();
		
	scrollBar = w->portRect;
	scrollBar.right -= 15; scrollBar.bottom -= 15;
	FrameRect(&scrollBar);
	
	CloseRgn(rgn);
	
	SetClip(rgn);
	DisposeRgn(rgn);
}
