/*
 * Menusx.c
 */

#include "Menusx.h"
#include "Alertsx.h"
#include "Scenario.h"
#include "Map Window.h"
#include "StrNUtil.h"
#include "WinManager.h"
#include "Scen Window.h"

void MenuBarInit (void)
// the slim, non-robust version
{
	Handle			menuBar;
	MenuHandle		menu;
	OSErr			err;
	
	menuBar = GetNewMBar(kMenuBarID);			// read menus into menu bar 
	if ( menuBar == nil ) {
		DoError("\pCan't load the menu bar!",0,kFatal);
		ExitToShell();
	}
	
	SetMenuBar(menuBar);					// install menus 
	DisposHandle(menuBar);
	AddResMenu(GetMHandle(mApple), 'DRVR');	// add DA names to Apple menu 

	InsertMenu(GetMenu(mScenFiles),-1);

	DrawMenuBar();
}

void DrawModalCheck(short menu, short item)
{
	MenuHandle menuHandle;
	short i;
	
	menuHandle = GetMenu(menu);
	for (i = 0; i<CountMItems(menuHandle); i++) {
		CheckItem(menuHandle,i+1,false);
	}
	CheckItem(menuHandle,item,true);
}

void UpdateFileMenu(void)
{
	MenuHandle menu = GetMenu(mFile);
	WindowPtr w = FrontWindow();
	long type;
				
	if (!menu) return;
	
	// Close Item.
	if (w) {
		EnableItem(menu,iClose);
	} else {
		DisableItem(menu,iClose);
	}
	
	// Saving
	if (!gScenario.file) {
		DisableItem(menu,iSave);
		DisableItem(menu,iSaveAs);
		DisableItem(menu,iSaveCopy);
		DisableItem(menu,iRevert);
	} else {
		EnableItem(menu,iSave);
		EnableItem(menu,iRevert);
		if (gScenario.scenType == kScenTypeFile) {
			EnableItem(menu,iSaveAs);
			EnableItem(menu,iSaveCopy);
		} else {
			DisableItem(menu,iSaveAs);
			DisableItem(menu,iSaveCopy);
		}
	}
	
	// Printing
	type = FindWindowType(w);
	if (type == kOutdoorsMap) {
		//EnableItem(menu,iPrint);
	} else {
		DisableItem(menu,iPrint);
	}
}

void BuildStandingOnMenu(void)
{
	short i,cnt;
	MenuHandle menuHandle;
	Str255 title;
	
	menuHandle = GetMenu(mStandingOn);
	if (!menuHandle) return; // error
	
	cnt = CountMItems(menuHandle);
	
	for (i=0; i<cnt; i++) DelMenuItem(menuHandle, 1); // erase menu
	
	MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,1); AppendMenu(menuHandle,title);
	MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,2); AppendMenu(menuHandle,title);
	MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,3); AppendMenu(menuHandle,title);
	MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,4); AppendMenu(menuHandle,title);
	MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,9); AppendMenu(menuHandle,title);
}

void BuildTileTextMenu(void)
{
	short i,cnt,strCnt;
	MenuHandle menuHandle;
	Str255 title;
	
	menuHandle = GetMenu(mTilesText);
	if (!menuHandle) return; // error
	
	cnt = CountMItems(menuHandle);
	
	for (i=0; i<cnt; i++) DelMenuItem(menuHandle, 1); // erase menu
	
	strCnt = *(short*)(*gScenario.strings[kSTRNTileTypes].strn);
	
	for (i=0; i<strCnt; i++) {
		MyGetIndString(title,gScenario.strings[kSTRNTileTypes].strn,i+1); AppendMenu(menuHandle,title);
	}
}

void BuildWindowMenu(void)
{
	long i,cnt;
	MenuHandle menuHandle;
	Str255 title;
	WindowPtr w;
	
	menuHandle = GetMenu(mWindow);
	if (!menuHandle) return; // error
	
	// first, rebuild menu.
	cnt = CountMItems(menuHandle);
	for (i=iWindow0; i<=cnt; i++) DelMenuItem(menuHandle, iWindow0); // erase menu

	cnt=CountWindows();
	for (i=0; i<cnt; i++) {
		GetWTitle(FindNthWindow(i+1), title); 
		AppendMenu(menuHandle, title);
		if (i<9) {
			NumToString(i+1,title);
			SetItemCmd(menuHandle, iWindow0 + i, title[1]); 
		}
	}
	
	if (cnt > 0) {
		EnableItem(menuHandle,iStack);
		EnableItem(menuHandle,iTile);
		EnableItem(menuHandle,iTileVert);
	} else {
		DisableItem(menuHandle,iStack);
		DisableItem(menuHandle,iTile);
		DisableItem(menuHandle,iTileVert);
	}
	
	// second, figure out if we disable menu.
	if ((FrontWindow() == nil) && (ScenarioListEmpty())) {
		DisableItem(menuHandle,0);
	} else {
		EnableItem(menuHandle,0);
	}
	
	// lastly, figure out if we dim the Scen Window option.
	if (gScenWindow) {
		EnableItem(menuHandle,iScenWindow);
	} else {
		DisableItem(menuHandle,iScenWindow);
	}
}

void BuildCityMenu (void)
{
	short i,cnt;
	MenuHandle menuHandle;
	Str255 title;
	
	menuHandle = GetMenu(mCities);
	if (!menuHandle) return; // error
	
	cnt = CountMItems(menuHandle);
	
	for (i=0; i<cnt; i++) DelMenuItem(menuHandle, 1); // erase menu
	
	for (i = 0; i<kNumberOfCityMaps; i++) {
		AppendMenu(menuHandle,gScenario.outsideMaps[i].mapName);
	}
}

void BuildDungeonMenu (void)
{
	short i,cnt;
	MenuHandle menuHandle;
	Str255 title;
	
	menuHandle = GetMenu(mDungeons);
	if (!menuHandle) return; // error
	
	cnt = CountMItems(menuHandle);
	
	for (i=0; i<cnt; i++) DelMenuItem(menuHandle, 1); // erase menu
	
	for (i = 0; i<kNumberOfDungeonMaps; i++) {
		AppendMenu(menuHandle,gScenario.dungeonMaps[i].mapName);
	}
}
