/*
 * Scenario.c
 *
 * Scenario File Handling routines.
 */

#include "Scen Window.h"
#include "Map Window.h"
#include "Scenario.h"
#include "Tiles.h"
#include "WatchCursor.h"
#include "Menusx.h"
#include "LinkedList.h"
#include "MMR.h"
#include "Progress.h"

#define kMapsIDUpperBound	421
#define kMapsIDLowerBound	400

#define kTotalResNum (kNumberOfOutsideMaps + kNumberOfDungeonMaps + kNumberOfOutsideMaps + kNumberOfTLKSEntries + kNumberOfMiscEntries + kNumberOfStrNEntries)

Boolean SelectUltimaApp(FSSpec *fs);
pascal Boolean GetScenarioFileFilter(CInfoPBPtr pb);
Boolean SelectGraphicsFile(FSSpec *fs);
pascal Boolean GetGraphicsFileFilter(CInfoPBPtr pb);
void AddScenarioToList(FSSpec *spec);


OSType	gScenarioFileType = 'scen';
OSType	gGraphicsFileType = 'grsc';
OSType  gApplCreator = 'U3se';
Scenario gScenario;
LinkedListPtr gScenFileList = nil;

// the following are hardcoded for now. When Leon comes up with something better,
// then I'll program it too.
// Maybe I should move all these into a resource ?!? -MDT 1/17/96
short gOutsideMapIDs[] = { 400,401,402,403,404,405,406,407,408,409,410,411,420,421 };
short gDungeonMapIDs[] = { 412,413,414,415,416,417,418 };
short gTLKSIDs[] = { 400,401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,416,417,418,421 };
short gMiscIDs[] = { 400,401,402,403,404,405,500,501,502,503,504,505 };
short gStringIDs[] = { 400,401,402,404,405,406,407,408,409,410,411,412,413,414 };

/* SelectScenarioFile
 * Opens StandardGetFile to get a scenario file. Called by
 */
Boolean SelectScenarioFile(FSSpec *fs)
{
	SFTypeList typeList;
	StandardFileReply reply;
	OSErr	err;
	FileFilterUPP upp;

	typeList[0] = gScenarioFileType;
	typeList[1] = 'APPL'; // we can always modify Ultima directly
	
	upp = NewFileFilterProc(GetScenarioFileFilter);
	StandardGetFile(upp,2, typeList, &reply);
	DisposeRoutineDescriptor(upp);
	
	if (reply.sfGood) *fs = reply.sfFile;

	return reply.sfGood;
}

Boolean SelectUltimaApp(FSSpec *fs)
{
	SFTypeList typeList;
	StandardFileReply reply;
	OSErr	err;
	FileFilterUPP upp;

	typeList[0] = 'APPL'; 
	
	upp = NewFileFilterProc(GetScenarioFileFilter);
	StandardGetFile(upp,1, typeList, &reply);
	DisposeRoutineDescriptor(upp);
	
	if (reply.sfGood) *fs = reply.sfFile;

	return reply.sfGood;
}

pascal Boolean GetScenarioFileFilter(CInfoPBPtr pb)
{
	FileParam *fileParam = (FileParam *)&(pb->hFileInfo);
		
	if (fileParam->ioFlFndrInfo.fdType == 'APPL') {
		if (fileParam->ioFlFndrInfo.fdCreator == 'Ult3') {
			return false;
		}
		return true;
	}
	return false;
}


/* SelectGraphicsFile
 * Opens StandardGetFile to get a graphics file. Called by
 */
Boolean SelectGraphicsFile(FSSpec *fs)
{
	SFTypeList typeList;
	StandardFileReply reply;
	OSErr	err;
	FileFilterUPP upp;

	typeList[0] = gScenarioFileType;
	typeList[1] = gGraphicsFileType;
	
	upp = NewFileFilterProc(GetGraphicsFileFilter);
	StandardGetFile(upp,2, typeList, &reply);
	DisposeRoutineDescriptor(upp);
	
	if (reply.sfGood) *fs = reply.sfFile;

	return reply.sfGood;
}

pascal Boolean GetGraphicsFileFilter(CInfoPBPtr pb)
{
	FileParam *fileParam = (FileParam *)&(pb->hFileInfo);
		
	if (fileParam->ioFlFndrInfo.fdType == gGraphicsFileType) {
		if (fileParam->ioFlFndrInfo.fdCreator == 'Ult3') {
			return false;
		}
		return true;
	}
	return false;
}

/* OpenScenarioFile
 * Opens Scenario File, copying scenario resources resource to the scenario record.
 */
OSErr OpenScenarioFile(FSSpec *fileSpec, Boolean opt)
{
	OSErr	err = noErr;
	short 	refNum,curResFile,i;
	Handle handle;
	short	dmyShort;
	ResType	dmyResType;
	FSSpec grafSpec,spec;
	FInfo	fInfo;
	long	length;
	short cnt;
	
	DoWatchCursor(FrontWindow(),kWatchOn);
	StartProgress(200, "\pOpening Scenario File");
	
	spec = *fileSpec;

	// first, find out if we're opening an 'APPL' or a 'scen'
	if (err = FSpGetFInfo(&spec, &fInfo)) goto bail;
	if (fInfo.fdType == 'scen')
		gScenario.scenType = kScenTypeFile;
	else if (fInfo.fdType == 'APPL')
		gScenario.scenType = kScenTypeAPPL;
	else 
		gScenario.scenType = -1;

	// Find our App's Res File ID
	curResFile = CurResFile();
	
	// Open files's Res Fork
	refNum = FSpOpenResFile(&spec, fsRdPerm);
	if (refNum == -1) goto bail;
	
	// Set files's Res Fork as our own
	UseResFile(refNum);
	if ((err = ResError()) != noErr) goto bail;
	
	// Get prefs & other general settings.
	ChangeCaption("\pReading Version Info");
	gScenario.scenVersion = ReadVersion();
	
	// Get the Outside Maps
	ChangeCaption("\pReading Scenario Data");
	UpdateProgress(0);
	cnt = 0;
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		handle = Get1Resource('MAPS', gOutsideMapIDs[i]);
		if (err = ResError()) goto bail;		
		// get the name
		GetResInfo(handle,&dmyShort, &dmyResType,(gScenario.outsideMaps[i].mapName));
		
		// set map data; incl. dimensions info.
		if (gScenario.scenVersion > kUltima12) {
			length = GetHandleSize(handle)-1; // 1 byte for length info
			gScenario.outsideMaps[i].map = NewHandle(length); 
			HLock(gScenario.outsideMaps[i].map); HLock(handle);
			BlockMove((*handle)+1,*(gScenario.outsideMaps[i].map), length);
			
			length = *(unsigned char*)*handle; if (length == 0) length = 256;
			gScenario.outsideMapsSize[i].h = gScenario.outsideMapsSize[i].v = length;
			HUnlock(gScenario.outsideMaps[i].map); HUnlock(handle);
		} else {
			DetachResource(handle);
			gScenario.outsideMaps[i].map = handle;
			gScenario.outsideMapsSize[i].h = gScenario.outsideMapsSize[i].v = 0x40; // 0x40 = 64 = size of map
		}
		// set map id.
		gScenario.outsideMaps[i].mapID = gOutsideMapIDs[i];	
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// Get the Dungeon Maps
	for (i=0; i<kNumberOfDungeonMaps; i++) {
		handle = Get1Resource('MAPS', gDungeonMapIDs[i]);
		if (err = ResError()) goto bail;			
		GetResInfo(handle,&dmyShort, &dmyResType,(gScenario.dungeonMaps[i].mapName));

		DetachResource(handle);
		gScenario.dungeonMaps[i].map = handle;
		gScenario.dungeonMaps[i].mapID = gDungeonMapIDs[i];	

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// Get the 'MONS' data
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		handle = Get1Resource('MONS', gOutsideMapIDs[i]);
		if (err = ResError()) goto bail;		
		// no name

		DetachResource(handle);
		gScenario.monsters[i].mons = handle;
		gScenario.monsters[i].monsID = gOutsideMapIDs[i];	

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// Get the 'TLKS' data
	for (i=0; i<kNumberOfTLKSEntries; i++) {
		handle = Get1Resource('TLKS', gTLKSIDs[i]);
		if (err = ResError()) goto bail;		
		GetResInfo(handle,&dmyShort, &dmyResType,(gScenario.dialogue[i].tlksName));
		
		DetachResource(handle);
		gScenario.dialogue[i].tlks = handle;
		gScenario.dialogue[i].tlksID = gTLKSIDs[i];	

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// Get the 'MISC' data
	for (i=0; i<kNumberOfMiscEntries; i++) {
		handle = Get1Resource('MISC', gMiscIDs[i]);
		if (err = ResError()) goto bail;		
		GetResInfo(handle,&dmyShort, &dmyResType,(gScenario.misc[i].miscName));
		
		DetachResource(handle);
		gScenario.misc[i].misc = handle;
		gScenario.misc[i].miscID = gMiscIDs[i];	

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// Get 'STR#' data
	for (i=0; i<kNumberOfStrNEntries; i++) {
		handle = Get1Resource('STR#', gStringIDs[i]);
		if (err = ResError()) goto bail;		
		GetResInfo(handle,&dmyShort, &dmyResType,(gScenario.strings[i].strnName));
		
		DetachResource(handle);
		gScenario.strings[i].strn = handle;
		gScenario.strings[i].strnID = gStringIDs[i];	

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}

	
	// Now load the Graphics:
	ChangeCaption("\pLoading Graphics");
	UpdateProgress(0);
	if ((opt) && (SelectGraphicsFile(&grafSpec))){
		CloseResFile(refNum); // close scenario file.
		refNum = FSpOpenResFile(&grafSpec, fsRdPerm); // open graphics file
		if (refNum == -1) goto bail;
		UseResFile(refNum);	 // set the res fork.
		if ((err = ResError()) != noErr) goto bail;
		err = LoadTiles(&gScenario.tiles);
		UseResFile(curResFile);
	} else { // use default
		UseResFile(curResFile);
		err = LoadTiles(&gScenario.tiles);				
	}
	
	// create backups now that we've closed off other files.
	cnt=0;
	ChangeCaption("\pCreating Backups");
	UpdateProgress(0);
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		handle = gScenario.outsideMaps[i].map;
		HandToHand(&handle);
		gScenario.outsideMaps[i].mapCopy = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfDungeonMaps; i++) {
		handle = gScenario.dungeonMaps[i].map;
		HandToHand(&handle);
		gScenario.dungeonMaps[i].backup = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		handle = gScenario.monsters[i].mons;
		HandToHand(&handle);
		gScenario.monsters[i].backup = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfTLKSEntries; i++) {
		handle = gScenario.dialogue[i].tlks;
		HandToHand(&handle);
		gScenario.dialogue[i].backup = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfMiscEntries; i++) {
		handle = gScenario.misc[i].misc;
		HandToHand(&handle);
		gScenario.misc[i].backup = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfStrNEntries; i++) {
		handle = gScenario.strings[i].strn;
		HandToHand(&handle);
		gScenario.strings[i].backup = handle;
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// reset dirty flag
	gScenario.scenDirty = false;

bail:	// If an error occurs, jump here
	ChangeCaption("\pFinishing Up");
	UpdateProgress(100);

	// Close the file. Ignore any error from this
	CloseResFile(refNum);

	if (err == noErr) {
		if (gScenario.file) DisposePtr((Ptr)gScenario.file);
		gScenario.file = (FSSpec*)NewPtr(sizeof(FSSpec));
		*gScenario.file = spec;
	}

	EndProgress();
	DoWatchCursor(FrontWindow(),kWatchOff);
	return err;
}

/* SaveScenarioFile
 * Opens Scenario File, copying whole MAPS resource to the App's resource map.
 */
OSErr SaveScenarioFile(FSSpec *spec)
{
	OSErr	err = noErr;
	short 	refNum,curResFile,i;
	Handle handle,sVerHandle=nil;
	FInfo	fInfo;
	short cnt = 0;
	
	DoWatchCursor(FrontWindow(),kWatchOn);
	StartProgress(200,"\pSaving Scenario File");

	// if we're opening a 'scen', build the Sver resource handle.
	if (err = FSpGetFInfo(spec, &fInfo)) goto bail;
	if (fInfo.fdType == 'scen') {
		gScenario.scenType = kScenTypeFile;
		sVerHandle = NewHandle(8); // sizeof sver handle
		((unsigned long*)*sVerHandle)[0] = ReadVersion();
		((unsigned long*)*sVerHandle)[1] = gScenario.scenVersion;
	}

	// Find our App's Res File ID
	curResFile = CurResFile();
	
	// Open files's Res Fork
	refNum = FSpOpenResFile(spec, fsWrPerm);
	if (refNum == -1) goto bail;
	
	// Set files's Res Fork as our own
	UseResFile(refNum);
	if ((err = ResError()) != noErr) goto bail;
	
	// save misc
	
	// if we're saving a scenario file, save the 'Sver' resource
	ChangeCaption("\pWriting out Version Info");
	if (sVerHandle) {
		// erase original copy.
		handle = Get1Resource('Sver', 128);
		if (handle) {
			RemoveResource(handle);
			DisposeHandle(handle);
		}
		// save new copy
		AddResource(sVerHandle,'Sver',128,"\pScenario Version Info");
		WriteResource(sVerHandle);
		if ((err = ResError()) != noErr) goto bail;	
		DetachResource(sVerHandle);		// can I replace the above lines with ReleaseResource 
		DisposeHandle(sVerHandle);
	}
	
	ChangeCaption("\pSaving Map Data");
	UpdateProgress(0); cnt= 0;
	// save the outside maps data
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		// erase original copy.
		handle = Get1Resource('MAPS', gOutsideMapIDs[i]);
		SetResAttrs(handle,0);
		RemoveResource(handle);
		DisposeHandle(handle);
		
		// build new handle
		if (gScenario.scenVersion > kUltima12) {
			handle = NewHandle(1);								// make new handle.
			**handle = gScenario.outsideMapsSize[i].h;			// set size info.
			HandAndHand(gScenario.outsideMaps[i].map,handle);	// copy map data.
		} else {
			handle = gScenario.outsideMaps[i].map;
			HandToHand(&handle);									// copy map data
		}
		
		// write out resource
		AddResource(handle,'MAPS',
				gScenario.outsideMaps[i].mapID,
				gScenario.outsideMaps[i].mapName);
		WriteResource(handle);
		if ((err = ResError()) != noErr) goto bail;	
		SetResAttrs(handle, resLocked | resProtected);
		//DetachResource(gScenario.outsideMaps[i].map);
		ReleaseResource(handle);
		
		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}		
	
	// save the dungeon maps data
	for (i=0; i<kNumberOfDungeonMaps; i++) {
		handle = Get1Resource('MAPS', gDungeonMapIDs[i]);
		SetResAttrs(handle,0);
		RemoveResource(handle);

		AddResource(gScenario.dungeonMaps[i].map,'MAPS',
				gScenario.dungeonMaps[i].mapID,
				gScenario.dungeonMaps[i].mapName);
				
		WriteResource(gScenario.dungeonMaps[i].map);
		if ((err = ResError()) != noErr) goto bail;	
		SetResAttrs(gScenario.dungeonMaps[i].map, resLocked | resProtected);
		DetachResource(gScenario.dungeonMaps[i].map);

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	
	// save the monster data
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		handle = Get1Resource('MONS', gOutsideMapIDs[i]);
		SetResAttrs(handle,0);
		RemoveResource(handle);

		AddResource(gScenario.monsters[i].mons,'MONS',
				gScenario.monsters[i].monsID,
				"\p");
				
		WriteResource(gScenario.monsters[i].mons);
		if ((err = ResError()) != noErr) goto bail;	
		SetResAttrs(gScenario.monsters[i].mons, resLocked);
		DetachResource(gScenario.monsters[i].mons);

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}		

	// save the TLKS data
	for (i=0; i<kNumberOfTLKSEntries; i++) {
		handle = Get1Resource('TLKS', gTLKSIDs[i]);
		SetResAttrs(handle,0);
		RemoveResource(handle);

		AddResource(gScenario.dialogue[i].tlks,'TLKS',
				gScenario.dialogue[i].tlksID,
				gScenario.dialogue[i].tlksName);
				
		WriteResource(gScenario.dialogue[i].tlks);
		if ((err = ResError()) != noErr) goto bail;	
		//SetResAttrs(gScenario.dialogue[i].tlks, resLocked); no atts for TLKS.
		DetachResource(gScenario.dialogue[i].tlks);

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}		

	// save the MISC data
	for (i=0; i<kNumberOfMiscEntries; i++) {
		handle = Get1Resource('MISC', gMiscIDs[i]);
		SetResAttrs(handle,0);
		RemoveResource(handle);

		AddResource(gScenario.misc[i].misc,'MISC',
				gScenario.misc[i].miscID,
				gScenario.misc[i].miscName);
				
		WriteResource(gScenario.misc[i].misc);
		if ((err = ResError()) != noErr) goto bail;	
		//SetResAttrs(gScenario.misc[i].misc, resLocked); no atts for MISC either.
		DetachResource(gScenario.misc[i].misc);

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}		

	// save the STR# data
	for (i=0; i<kNumberOfStrNEntries; i++) {
		handle = Get1Resource('STR#', gStringIDs[i]);
		if (handle) {
			SetResAttrs(handle,0);
			RemoveResource(handle);
		}

		AddResource(gScenario.strings[i].strn,'STR#',
				gScenario.strings[i].strnID,
				gScenario.strings[i].strnName);
				
		WriteResource(gScenario.strings[i].strn);
		if ((err = ResError()) != noErr) goto bail;	
		//SetResAttrs(gScenario.strings[i].strn, resLocked); no atts for STR# either.
		DetachResource(gScenario.strings[i].strn);

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}		
		
	// Reset Res File
	UseResFile(curResFile);
	
	ChangeCaption("\pRecreating Backups");
	UpdateProgress(0); cnt = 0;
	// create map backups now that we're using our own res fork.
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		if (gScenario.outsideMaps[i].mapCopy) DisposeHandle(gScenario.outsideMaps[i].mapCopy);
		handle = gScenario.outsideMaps[i].map;
		HandToHand(&handle);
		gScenario.outsideMaps[i].mapCopy = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfDungeonMaps; i++) {
		if (gScenario.dungeonMaps[i].backup) DisposeHandle(gScenario.dungeonMaps[i].backup);
		handle = gScenario.dungeonMaps[i].map;
		HandToHand(&handle);
		gScenario.dungeonMaps[i].backup = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		if (gScenario.monsters[i].backup) DisposeHandle(gScenario.monsters[i].backup);
		handle = gScenario.monsters[i].mons;
		HandToHand(&handle);
		gScenario.monsters[i].backup = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfTLKSEntries; i++) {
		if (gScenario.dialogue[i].backup) DisposeHandle(gScenario.dialogue[i].backup);
		handle = gScenario.dialogue[i].tlks;
		HandToHand(&handle);
		gScenario.dialogue[i].backup = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfMiscEntries; i++) {
		if (gScenario.misc[i].backup) DisposeHandle(gScenario.misc[i].backup);
		handle = gScenario.misc[i].misc;
		HandToHand(&handle);
		gScenario.misc[i].backup = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	for (i=0; i<kNumberOfStrNEntries; i++) {
		if (gScenario.strings[i].backup) DisposeHandle(gScenario.strings[i].backup);
		handle = gScenario.strings[i].strn;
		HandToHand(&handle);
		gScenario.strings[i].backup = handle;

		UpdateProgress((((float)(++cnt)/kTotalResNum) * 100));
	}
	gScenario.scenDirty = false;
	
bail:	// If an error occurs, jump here
	ChangeCaption("\pFinishing Up");
	UpdateProgress(100);

	if (err == noErr) err = ResError();

	// Close the file. Ignore any error from this
	CloseResFile(refNum);

	EndProgress();
	DoWatchCursor(FrontWindow(),kWatchOff);
	return err;
}

/* CloseScenarioFile
 * A misnomer - since we don't leave the files open, we don't have to close them.
 * However, we can use this to do cleanup once a "document" is "closed"; i.e., 
 * remove the sprite memory, etc etc. This will avoid mem-leaks.
 */
// fileSpec unused
void CloseScenarioFile(FSSpec *fileSpec)
{
	short i;

	// free the Outside Maps
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		DisposeHandle(gScenario.outsideMaps[i].map);
		DisposeHandle(gScenario.outsideMaps[i].mapCopy);
	}
	
	// free the Dungeon Maps
	for (i=0; i<kNumberOfDungeonMaps; i++) {
		DisposeHandle(gScenario.dungeonMaps[i].map);
		DisposeHandle(gScenario.dungeonMaps[i].backup);
	}
	
	// free the monsters
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		DisposeHandle(gScenario.monsters[i].mons);
		DisposeHandle(gScenario.monsters[i].backup);
	}
	
	// free the TLKS
	for (i=0; i<kNumberOfTLKSEntries; i++) {
		DisposeHandle(gScenario.dialogue[i].tlks);
		DisposeHandle(gScenario.dialogue[i].backup);
	}

	// free the misc
	for (i=0; i<kNumberOfMiscEntries; i++) {
		DisposeHandle(gScenario.misc[i].misc);
		DisposeHandle(gScenario.misc[i].backup);
	}
	
	// free the strings
	for (i=0; i<kNumberOfStrNEntries; i++) {
		DisposeHandle(gScenario.strings[i].strn);
		DisposeHandle(gScenario.strings[i].backup);
	}
	
	DisposeTiles(&gScenario.tiles);
	if (gScenario.file != nil) {
		AddScenarioToList(gScenario.file);
		DisposePtr((Ptr)gScenario.file);
		gScenario.file = nil;
	}
}

/* SaveNewScenarioFile
 * Saves a new scenario file - used in Save As and Save A Copy As
 */

OSErr SaveNewScenarioFile(Boolean saveACopyOnly)
{
	FSSpec spec;
	StandardFileReply reply;
	OSErr err;
	short refNum;

	StandardPutFile("\pSave scenario as:", "\pUntitled", &reply);

	if (!reply.sfGood) return noErr;
	spec = reply.sfFile;
	
	if (!reply.sfReplacing) {
		(void) FSpCreateResFile(&spec, gApplCreator, gScenarioFileType, reply.sfScript);
		if (err = ResError()) return err;
	}

	err = SaveScenarioFile(&spec);
	if (err) return err;
	
	if (!saveACopyOnly) {
		//if (gScenario.file) DisposePtr((Ptr)gScenario.file);
		if (!gScenario.file) gScenario.file = (FSSpec*)NewPtr(sizeof(FSSpec));
		*gScenario.file = spec;
	}
	
	return err;
}

OSErr InstallScenarioFile(void)
{	
//	Boolean fileAlreadyLoaded=false;
	FSSpec scenario, app;
	OSErr err;
	
	//if (gScenWindow) fileAlreadyLoaded=true;
	
	if (!gScenWindow) {
		if (!SelectScenarioFile(&scenario)) return noErr;
		err = OpenScenarioFile(&scenario,false);
		if (err) return err;
	}
	
	if (!SelectUltimaApp(&app)) return noErr;
	err = SaveScenarioFile(&app);
	//if (err) goto installFail;
	
	if (!gScenWindow) {
		CloseScenarioFile(&scenario); // skip this error condition
	}
	
	return err;
}

OSErr ExtractScenarioFile(void)
{
	FSSpec /*scenario*/ app;
	OSErr err;

	if (gScenWindow) {
		if (CloseScenWindow()) { 
			//CloseAllMapWindows(); // prevent problems!
			ForEachMapWindowDo(CloseMapWindow); 
		} else {
			return noErr;
		}
	}
	
	if (!SelectUltimaApp(&app)) return noErr;
	//if (!SelectScenarioFile(&scenario)) return noErr;
	
	err = OpenScenarioFile(&app,false);
	if (err) return err;

	//err = SaveScenarioFile(&scenario);
	err = SaveNewScenarioFile(false);
	if (err) {
		CloseScenarioFile(&app);	
		return err;
	}
	
	//*gScenario.file = scenario; // set in savenewscenfile
	return noErr;
}

void RevertOutsideMap(short mapID)
{
	Handle handle;
	
	DisposeHandle(gScenario.outsideMaps[mapID].map);
	handle = gScenario.outsideMaps[mapID].mapCopy;
	HandToHand(&handle);
	gScenario.outsideMaps[mapID].map = handle;
}

void RevertMonsters(short mapID)
{
	Handle handle;
	
	DisposeHandle(gScenario.monsters[mapID].mons);
	handle = gScenario.monsters[mapID].backup;
	HandToHand(&handle);
	gScenario.monsters[mapID].mons = handle;
}

void RevertDungeonMaps(short mapID)
{
	Handle handle;
	
	DisposeHandle(gScenario.dungeonMaps[mapID].map);
	handle = gScenario.dungeonMaps[mapID].backup;
	HandToHand(&handle);
	gScenario.dungeonMaps[mapID].map = handle;
}

void RevertTLKSEntries(short i)
{
	Handle handle;
	
	DisposeHandle(gScenario.dialogue[i].tlks);
	handle = gScenario.dialogue[i].backup;
	HandToHand(&handle);
	gScenario.dialogue[i].tlks = handle;
}
void RevertMiscEntries(short i)
{
	Handle handle;
	
	DisposeHandle(gScenario.misc[i].misc);
	handle = gScenario.misc[i].backup;
	HandToHand(&handle);
	gScenario.misc[i].misc = handle;
}
void RevertStrNEntries(short i)
{
	Handle handle;
	
	DisposeHandle(gScenario.strings[i].strn);
	handle = gScenario.strings[i].backup;
	HandToHand(&handle);
	gScenario.strings[i].strn = handle;
}


void RevertAll(void)
{
	short i;
	for (i=0; i<kNumberOfOutsideMaps; i++) {
		RevertOutsideMap(i);
		RevertMonsters(i); // also uses Outside Map IDs.
	}
	for (i=0; i<kNumberOfDungeonMaps; i++) RevertDungeonMaps(i);
	for (i=0; i<kNumberOfTLKSEntries; i++) RevertTLKSEntries(i);
	for (i=0; i<kNumberOfMiscEntries; i++) RevertMiscEntries(i);
	for (i=0; i<kNumberOfStrNEntries; i++) RevertStrNEntries(i);

	gScenario.scenDirty = false;
}

void AddScenarioToList(FSSpec *spec)
{
	short i,count;
	FSSpec *f;
	MenuHandle menu;
	
	if (!spec) return;
	if (!gScenFileList) NewLinkedList(&gScenFileList);
	
	count = LinkedListCount(gScenFileList);
	for (i=0; i<count;i++) {
		f = (FSSpec *) GetListItemByRank(gScenFileList,i+1);
		if (EqualFSSpec(f,spec)) return; // already in list
	}
	f = (FSSpec *)NewPtr(sizeof(FSSpec));
	if (!f) return;
	*f = *spec;
	AddItemToList(gScenFileList,(long)f,0L);
	
	// build scenario window. Should I move this to Menusx?
	menu = GetMenu(mScenFiles);
	if (!menu) return;
	AppendMenu(menu,f->name);
	EnableItem(GetMenu(mWindow),iScenSwitch);
}

Boolean ScenarioListEmpty(void)
{
	return LinkedListIsEmpty(gScenFileList);
}

#include "TileMenuCommon.h"
OSErr OpenScenarioFromList(short scenNum)
{
	FSSpec *f = nil;
	OSErr err;
	
	f = (FSSpec *)GetListItemByRank(gScenFileList,scenNum);
	err = OpenScenarioFile(f,false);
	if (err == noErr) {
		SetTMTiles(gScenario.tiles);
		CheckTMItem(1);
		(void)NewScenWindow();
		UpdateFileMenu();
	}
	return err;
}

/* ReadVersion attempts to read the version number for a specific scenario. If the
 * open resource file has a 'vers' resource, ReadVersion returns the first 4 bytes
 * of 'vers' 1; generally, this will be an Ultima III application. IF the 'vers'
 * resource does not exist, ReadVersion tries to read the 2nd long word from 'Sver'
 * 128. The 'Sver' resource documents the version number of both the Ultima III
 * Editor program that created the file and the Ultima III program it was derived
 * from. If it can find neither of these things (i.e., it's an old scenario), it
 * assumes version 1.2.0f0. 
 */
unsigned long ReadVersion(void)
{
	Handle versHdl;
	unsigned long vers;
	
	// read vers resource
	versHdl = Get1Resource('vers',1);
	if (versHdl) { // read the version info.
		vers = *((long*)*versHdl);
	} else if (versHdl == nil) { // if no vers, try Sver
		versHdl = Get1Resource('Sver',128);
		if (versHdl) { // read the version info.
			vers = ((unsigned long*)(*versHdl))[1];
		} else {
			vers = kUltima12; // 1.2.0f0, the standard release
		}
	}
	if (versHdl) ReleaseResource(versHdl);
	
	return (vers);
}