/*
 * Scenario.h
 */

#ifndef __SCENARIO__
#define __SCENARIO__

#include <QDOffscreen.h>

#define kNumberOfOutsideMaps	14
#define kNumberOfDungeonMaps	7
#define kNumberOfTLKSEntries	20
#define kNumberOfMiscEntries	12
#define kNumberOfStrNEntries	14
#define kNumberOfCityMaps		12 // mdt-added later

#define kScenTypeFile			0
#define kScenTypeAPPL			1

#define kSTRNTileTypes			0

#define kUltima12				0x01208000

struct OutsideMapsRec {
	short mapID;
	Str63 mapName;
	Handle map;
	Handle mapCopy; // for our eraser.
};
typedef struct OutsideMapsRec OutsideMapsRec;

struct DungeonMapsRec {
	short mapID;
	Str63 mapName;
	Handle map;
	Handle backup;
};
typedef struct DungeonMapsRec DungeonMapsRec;

struct MonsRec {
	short monsID;
	Handle mons;
	Handle backup;
};
typedef struct MonsRec MonsRec;

struct TLKSRec {
	short tlksID;
	Str63 tlksName;
	Handle tlks;
	Handle backup;
};
typedef struct TLKSRec TLKSRec;

struct MiscRec {
	short miscID;
	Str63 miscName;
	Handle misc;
	Handle backup;
};
typedef struct MiscRec MiscRec;

struct StrNRec {
	short strnID;
	Str63 strnName;
	Handle strn;
	Handle backup;
};
typedef struct StrNRec StrNRec;

struct ScenarioPrefs { // 'ScPr' ? 
	Str63	divTitles[4];
	Boolean	divOpen[4];
	long	appVersion;
	long	edVersion;
};
typedef struct ScenarioPrefs ScenarioPrefs;

struct Scenario {
	// resource structs:
	OutsideMapsRec	outsideMaps[kNumberOfOutsideMaps]; // later, this will become a linked list;
	Point			outsideMapsSize[kNumberOfOutsideMaps]; // for variable sized maps.
	DungeonMapsRec	dungeonMaps[kNumberOfDungeonMaps]; 
	MonsRec			monsters[kNumberOfOutsideMaps];
	TLKSRec			dialogue[kNumberOfTLKSEntries];
	MiscRec			misc[kNumberOfMiscEntries];
	StrNRec			strings[kNumberOfStrNEntries];
	
	// other scenario details:	
	ScenarioPrefs	prefs;
	FSSpec			*file;
	GWorldPtr 		tiles;
	char			scenType;
	Boolean			scenDirty;
	unsigned long	scenVersion;
};
typedef struct Scenario Scenario;


extern  Scenario gScenario;

Boolean SelectScenarioFile(FSSpec *spec);
OSErr OpenScenarioFile(FSSpec *spec, Boolean opt);
OSErr SaveScenarioFile(FSSpec *spec);
void CloseScenarioFile(FSSpec *spec);
OSErr SaveNewScenarioFile(Boolean saveACopyOnly);
OSErr InstallScenarioFile(void);
OSErr ExtractScenarioFile(void);
void RevertOutsideMap(short mapID);
void RevertMonsters(short mapID);
void RevertDungeonMaps(short mapID);
void RevertTLKSEntries(short i);
void RevertMiscEntries(short i);
void RevertStrNEntries(short i);
void RevertAll(void);
OSErr OpenScenarioFromList(short scenNum);
unsigned long ReadVersion(void);
Boolean ScenarioListEmpty(void);


#endif /* __SCENARIO__ */