/*
 * TileInfoDLOG.c
 */
/* 5/15/96 - start conversion to multi-pane dialog box.
 */

#include "Alertsx.h"
#include "TileInfo.h"
#include "Tiles.h"
#include "Scenario.h"
#include "Menusx.h"
#include "StrNUtil.h"
#include "TileMenuCommon.h"
#include "DLOGUtil.h"
#include "MMR.h"

#include "MultiPane.h" // MDT 5/15/96
#include "Monsters.h"

#include "Locations.h" // MDT 5/26/96

//#define kTileInfoDLOGID		132
#define kTileInfoDLOGID		150

#define kOK					1
#define kCancel				2
#define kTile				3
#define kTileName			4
#define kItemType			5
#define kTilePos			6
#define kLine				7
#define kCustomErase		12

#define kMnPnStandingOn		2
#define kMnPnTLKS			3
#define	kMnPnDisposition	4

#define kLocID				1

#define kBadTileDLOGID		133
#define kBadRangeDLOGID		134

#define tiNormal	0
#define tiMonster	1
#define tiCity		2
#define tiDungeon	3
#define tiMoongate	4

void LoadTileInfoDLOG(DialogPtr d);
void DrawControlGroup(Rect controlGroup, Rect caption);
void DrawTileInfoDLOG(DialogPtr d);
pascal Boolean TileInfoKeyProc(DialogPtr d, EventRecord *e, short *itemHit);
void SaveTileInfoDLOG(DialogPtr d,short mapID);
void TIDrawTilePopup(DialogPtr d);
Boolean TIIsTilePopupClick(DialogPtr d,Point localPt);
void DoBadTileDialog (void);
void DoBadRangeDialog (Str255 itemName, long max);
void SetTITileType(short mapID, Point mapPt);


struct TileInfoRec {
	DialogPtr	d;
	short tile;
	short tileType; // i.e., monster, normal, etc.
	Point tilePt;
	long  mapSize;
	union {
		MonsterRec monster;
		short 		city;
		short		dungeon;
		short		moongate;
	}data;
};
typedef struct TileInfoRec TileInfoRec;


TileInfoRec gTileInfo;

void DoTileInfoDLOG(short mapID, Point mapPt)
{
	DialogPtr d;
	Boolean done = false;
	GrafPtr save;
	ModalFilterUPP upp;
	short item, pane, savedTMI;
	unsigned char *a; 
	Str255 str;
	Handle handle;
	
	a = (unsigned char *)*gScenario.outsideMaps[mapID].map;
	
	savedTMI = GetTMCheckedItem();  // get current tile from Tile Popup.

	//gather information about the tile that's been clicked on.
	//   basics
	gTileInfo.mapSize = gScenario.outsideMapsSize[mapID].h;
	gTileInfo.tile = a[(gTileInfo.mapSize*mapPt.v) + mapPt.h];
	gTileInfo.tileType = tiNormal;
	gTileInfo.tilePt = mapPt;
	CheckTMItem((gTileInfo.tile/4)+1);
	
	SetTITileType(mapID, mapPt);
	
	// create our MP Dialog
	GetPort(&save);
	d = GetNewMPDialog(kTileInfoDLOGID, nil, (WindowPtr)-1L); // MDT 5/22/96
	if (!d) {
		DoError("\pCould not load TileInfoDLOG!",0,kNotFatal);
		return;
	}
	LoadTileInfoDLOG(d);
	ShowWindow(d);
	SetPort(d);
	(void) SetDialogDefaultItem(d,1);
	(void) SetDialogCancelItem(d,2);
	SetMPCustomEraseItem(d,kCustomErase);
	
	gTileInfo.d = d;
	
	upp = NewModalFilterProc(TileInfoKeyProc);

	while (!done) {
		ModalDialog(upp,&item);
		MPItemHit(d,&item,&pane);
		
		switch(pane){
		case -1: // main pane
			switch(item) {
			case 1:
				//save
				SaveTileInfoDLOG(d,mapID);
				//fall through
			case 2:
				done = true;
				break;
			case 5:
				{ Handle handle;
					GetDItem2(d,5,nil,&handle,nil);
					if (!handle) break;
					switch (GetCtlValue((ControlHandle)handle)) {
					case 1: // normal control
						SetMPDialogPane(d,-1);
						break;
					case 3: // monster
						BuildStandingOnMenu();			// build our standing on menu
						SetMPDialogPane(d,0);
						break;
					case 4: // City
						BuildCityMenu();
						SetMPDialogPane(d,1);
						break;
					case 5: // dungeon
						BuildDungeonMenu();
						SetMPDialogPane(d,2);
						break;
					case 6: // moongate
						SetMPDialogPane(d,3);
						break;
					default:
						SetMPDialogPane(d,-1);
					}// switch menu			
				} // switch item
				break;
			default:
				;
			} // switch item
		case 0: // monsters
			break;
		default:
			;
		} // switch pane
	} // while
	
	CheckTMItem(savedTMI);
	DisposeRoutineDescriptor(upp);
	SetPort(save);
	DisposeMPDialog(d);
}

void SetTITileType(short mapID,Point mapPt)
{
	MonsterRec m;
	short id;

	//   is it a monster?
	FindMonsterByPosition(mapID, mapPt,&m);
	if (m.monsTileType > 0) {
		gTileInfo.tileType = tiMonster;
		gTileInfo.data.monster = m;
		return;
	}
	//   is it a city?
	id = FindLocationByPosition(kLocCity,mapPt);
	if (id != -1) {
		gTileInfo.tileType = tiCity;
		gTileInfo.data.city = id;
		return;
	}
	//   is it a dungeon?
	id = FindLocationByPosition(kLocDungeon,mapPt);
	if (id != -1) {
		gTileInfo.tileType = tiDungeon;
		gTileInfo.data.dungeon = id;
		return;
	}
	//   is it a Moongate?
	id = FindLocationByPosition(kLocMoongate,mapPt);
	if (id != -1) {
		gTileInfo.tileType = tiMoongate;
		gTileInfo.data.moongate = id;
		return;
	}
}

void LoadTileInfoDLOG(DialogPtr d)
{
	Str255 str,t;
	
	// set tile name
	MyGetIndString(str,gScenario.strings[0].strn,gTileInfo.tile/4 + 1);
	SetMPIText(d,str,kTileName,-1);

	// set tile location
	NumToString(gTileInfo.tilePt.h,str);
	PStringCat(str,"\p,");
	NumToString(gTileInfo.tilePt.v,t);
	PStringCat(str,t);
	SetMPIText(d,str,kTilePos,-1);

	switch (gTileInfo.tileType) {
	case tiNormal:
		SetMPDialogPane(d,-1);
		SetMPCtlValue(d,1,kItemType,-1);
		break;
	case tiMonster:
		BuildStandingOnMenu();	
		SetMPDialogPane(d,0);
		SetMPCtlValue(d,gTileInfo.data.monster.monsTileOnTopOf /4 + 1,kMnPnStandingOn,0);
		NumToString(gTileInfo.data.monster.monsTLKSID,str);
		SetMPIText(d,str,kMnPnTLKS,0);
		SetMPCtlValue(d,gTileInfo.data.monster.monsInclination / 4 + 1,kMnPnDisposition,0);
		SetMPCtlValue(d,3,kItemType,-1);
		break;
	case tiCity:
		BuildCityMenu();
		SetMPDialogPane(d,1);
		SetMPCtlValue(d,4,kItemType,-1);
		SetMPCtlValue(d,gTileInfo.data.city+1,kLocID,1);
		break;
	case tiDungeon:
		BuildDungeonMenu();
		SetMPDialogPane(d,2);
		SetMPCtlValue(d,5,kItemType,-1);
		SetMPCtlValue(d,gTileInfo.data.dungeon-12+1,kLocID,2);
		break;
	case tiMoongate:
		SetMPDialogPane(d,3);
		SetMPCtlValue(d,6,kItemType,-1);
		SetMPCtlValue(d,gTileInfo.data.moongate+1,kLocID,3);
		break;
	default:
		SetMPDialogPane(d,-1);
		SetMPCtlValue(d,1,kItemType,-1);
	}
}

void SaveTileInfoDLOG(DialogPtr d,short mapID)
{
	unsigned char *a; 
	Str255	str;
	long tmp;
	
	a = (unsigned char *)*gScenario.outsideMaps[mapID].map;
	
	// save new tile
	a[(gTileInfo.mapSize*gTileInfo.tilePt.v) + gTileInfo.tilePt.h] = gTileInfo.tile;
	
	switch (gTileInfo.tileType) {
	case tiMonster:
		gTileInfo.data.monster.monsTileOnTopOf = (GetMPCtlValue(d,kMnPnStandingOn,0)-1)*4;
		gTileInfo.data.monster.monsInclination = (GetMPCtlValue(d,kMnPnDisposition,0)-1)*4;
		GetMPIText(d,str,kMnPnTLKS,0);
		StringToNum(str,&tmp);
		gTileInfo.data.monster.monsTLKSID = tmp;
		
		SetMonster(mapID,&gTileInfo.data.monster);
		break;
	default:
		;
	}
}

/* TileInfoKeyProc
 * Based on AlertKeyProc.
 */
#define kEnterKey		3
#define kReturnKey		13
#define kEscapeKey		27
pascal Boolean TileInfoKeyProc(DialogPtr d, EventRecord *e, short *itemHit)
{
	char 	theChar;
	short	num, i, iType;
	Handle	iHandle;
	Rect	iRect;
	Str255	iText,iName;
	
	ModalFilterUPP	stdFilterProc;
	Boolean ret;
	OSErr	err;
	Point point;

	short	item;
	DialogRecord *dr;
	long size,maxEntry;

	switch(e->what) {
	case mouseDown:
		point = e->where;
		GlobalToLocal(&point);
		if (TIIsTilePopupClick(d,point)){
			MenuHandle menu;
			long menuChoice;
			
			GetDItem(d,kTile,&iType, &iHandle, &iRect);	
			InsetRect(&iRect, -1,-1);
			TMInvertItem2(&iRect);
			
			menu = GetMenu(200);
			InsertMenu(menu,-1);
			SetPt(&point, iRect.left,iRect.top);
			LocalToGlobal(&point);
			menuChoice = PopUpMenuSelect(menu, point.v, point.h, 1);
			DeleteMenu(200);
			TMInvertItem2(&iRect);
	
			if (HiWord(menuChoice) == mTiles) {
				menuChoice = (long)LoWord(menuChoice);
				if (menuChoice == 1) {
					DoBadTileDialog();
				} else {
					gTileInfo.tile = (menuChoice-1) * 4;
					CheckTMItem((short)menuChoice);
					TIDrawTilePopup(d);
					// Draw proper tile type
					// should become a popup menu soon.
					MyGetIndString(iText,gScenario.strings[0].strn,menuChoice);
					GetDItem(d,kTileName,&iType,&iHandle, &iRect);
					SetIText(iHandle,iText);
				}
			}
			return ;
		}		
		break;
	case updateEvt:
		//BeginUpdate(d);
		DrawTileInfoDLOG(d);
		//EndUpdate(d);
		break;
	case keyDown:
	case autoKey:
		theChar = (e->message & charCodeMask);
		
		// MDT 1/31/96 turn balloon help on/off
		if (theChar == 0x05) {
			(void) HMSetBalloons(!HMGetBalloons()); 
			return true;
		}
		
		if (theChar == kReturnKey || theChar == kEnterKey) {
			GetDItem(d, 1, &iType, &iHandle, &iRect);
			FlashButton((ControlHandle)iHandle);
			
			*itemHit = 1;
			return (true);
		} else if ( (theChar == kEscapeKey) ||
				   ((theChar == '.') && (e->modifiers & cmdKey) ))  {
			GetDItem(d, 2, &iType, &iHandle, &iRect);
			if (iType == ctrlItem + btnCtrl) {		/* If there is a Button */
				FlashButton((ControlHandle)iHandle);
			
				*itemHit = 2;
				return (true);
			} else {
				return (false);
			}
		} else {
			//if ((e->modifiers & cmdKey)) { /* Only work if command key is down */
			num = CountDITL(d);
			for (i=1; i<=num; i++) {
				GetDItem(d, i, &iType, &iHandle, &iRect);
				if (iType == ctrlItem + btnCtrl) {
					GetCTitle((ControlHandle)iHandle, iText);
					if (theChar == iText[1] || theChar == (char)(iText[1] + 32)){
						FlashButton((ControlHandle)iHandle);
						*itemHit = i;
						return (true);
					}	// if
				}	// if
			}	// for
			
			// else, barring all the above, make sure numbers are entered correctly.
			if (IsDlgControl(theChar)) break;
			
			dr = (DialogRecord *)d;
			item = dr->editField+1;
			MakeDLOGString(d,e,item,iText);
			
			// make sure every character is legal
			for (i=1; i<=iText[0]; i++) {
				if (! IsNumber(iText[i]))
					return AbortEvent(e,true);
			}
			
			StringToNum(iText,&size);
			
		/*	switch(item) {
			case kXPosition:
				maxEntry = mpSize-1;
				PStringCopy("\pX Position", iName);
				break;
			case kYPosition:
				maxEntry = mpSize-1;
				PStringCopy("\pY Position",iName);
				break;
			case kTLKSID:
				maxEntry = 15;
				PStringCopy("\pTLKS ID", iName);
				break;
			default:
				maxEntry = 0;
				PStringCopy("\p<Unknown>", iName);
			}
			if (size > maxEntry) {
				DoBadRangeDialog(iName,maxEntry);
				AbortEvent(e,false);
			}*/

			//}	// if
		} // else
		break;
	default:
		;
	}	// switch
	
	// call the standard filter proc
	return StdFilterProc(d,e,itemHit);
}

void DoBadTileDialog (void)
{
	DialogPtr d;
	GrafPtr save;
	Boolean done = false;
	ModalFilterUPP upp;
	short itemHit;
	
	SysBeep(3);
	
	GetPort(&save);
	d = GetNewDialog(kBadTileDLOGID, nil, (WindowPtr)-1L);
	if (!d) {
		DoError("\pCould not load Bad Tile Dialog!",0,kNotFatal);
		return;
	}
	ShowWindow(d);
	SetPort(d);
	(void) SetDialogDefaultItem(d,kOK);

	upp = NewModalFilterProc(AlertKeyProc);

	while (!done) {
		ModalDialog(upp,&itemHit);
		
		switch (itemHit) {
		case kOK: // ok
			done = true;
			break;
		default:
			;
		}
	}
	DisposeRoutineDescriptor(upp);
	SetPort(save);
	DisposeDialog(d);
}

void DoBadRangeDialog (Str255 itemName, long max)
{
	DialogPtr d;
	GrafPtr save;
	Boolean done = false;
	ModalFilterUPP upp;
	short itemHit;
	Str255 itemMax;
	
	NumToString(max,itemMax);
	
	SysBeep(3);
	
	ParamText(itemName,itemMax,"\p","\p");
	GetPort(&save);
	d = GetNewDialog(kBadRangeDLOGID, nil, (WindowPtr)-1L);
	if (!d) {
		DoError("\pCould not load Bad Tile Dialog!",0,kNotFatal);
		return;
	}
	ShowWindow(d);
	SetPort(d);
	(void) SetDialogDefaultItem(d,kOK);

	upp = NewModalFilterProc(AlertKeyProc);

	while (!done) {
		ModalDialog(upp,&itemHit);
		
		switch (itemHit) {
		case kOK: // ok
			done = true;
			break;
		default:
			;
		}
	}
	DisposeRoutineDescriptor(upp);
	SetPort(save);
	DisposeDialog(d);
}

/* DrawTileInfoDLOG
 * Should be called in UpdateEvents and whatnot.
 */
void DrawTileInfoDLOG(DialogPtr d)
{
	Rect caption,controlGroup,iRect;
	short iType;
	Handle iHandle;
	short maxItems;
	short pane;
	
	// draw tile popup
	TIDrawTilePopup(d);
	
	GetDItem(d,7,&iType, &iHandle, &iRect);
	FrameRect(&iRect);
	
	pane = GetMPDialogPane(d);
	if (pane == 0) { 
	// frame control group.
		maxItems = GetMPDialogItemCount(d);
		GetDItem(d,maxItems+5,&iType, &iHandle, &caption);
		GetDItem(d,maxItems+9,&iType, &iHandle, &controlGroup);
		DrawControlGroup(controlGroup,caption);
	}
	DrawDefaultButton(d);
}

/* Draws one of them nifty boxes with controls in it. Pay attention:
 *   controlGroup	- rect containing the bounds of the box to draw.
 *   caption		- rect containing the caption label.
 * NOTE: this routine doesn't ensure the label is positioned properly
 * within the controlGroup. That's YOUR job!!!
 */
void DrawControlGroup(Rect controlGroup, Rect caption)
{
	short tmp;
	
	MoveTo(controlGroup.left,controlGroup.top);
	tmp = caption.left - controlGroup.left;
	if (tmp < 0) tmp = 0;
	LineTo(controlGroup.left + tmp, controlGroup.top);
	tmp = controlGroup.right - caption.right;
	if (tmp < 0) tmp = 0;
	MoveTo(controlGroup.right - tmp,controlGroup.top);
	LineTo(controlGroup.right,controlGroup.top);
	LineTo(controlGroup.right,controlGroup.bottom);
	LineTo(controlGroup.left,controlGroup.bottom);
	LineTo(controlGroup.left,controlGroup.top);
}

/* The following were stolen from Map Window.c */
void TIDrawTilePopup(DialogPtr d)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	
	// draw tile
	GetDItem(d,kTile,&iType, &iHandle, &iRect);	
	//   plot tile in iRect;
	PlotTile(gScenario.tiles,(short)gTileInfo.tile,nil,&iRect);
	//   draw frame
	InsetRect(&iRect,-2,-2);
	FrameRect(&iRect);
}

Boolean TIIsTilePopupClick(DialogPtr d,Point localPt)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	
	// draw tile
	GetDItem(d,kTile,&iType, &iHandle, &iRect);	
	InsetRect(&iRect, -2,-2);
	
	return (PtInRect(localPt, &iRect));
}

