/*
 * TileMenuCommon.c
 *
 * Common routines for the Tile menu
 */

#include <QDOffscreen.h>

#include "Menusx.h"
#include "TileMenuCommon.h"
#include "Tiles.h"

void TileMenuDraw(GWorldPtr tiles, Rect *mRect)
{
	short x,y,i;
	Rect r;

	// draw our frame:
	//   horizontal lines:
	for (i=0; i < kMenuHeight + 1; i++) {
		MoveTo(mRect->left + kMenuBorder, (mRect->top + kMenuBorder) + i*(kMItemHeight + 1));
		Line((kMItemHeight * kMenuHeight) + kMenuHeight,0);
	}
	//   vertical lines:
	for (i=0; i < kMenuWidth + 1; i++) {
		MoveTo((mRect->left + kMenuBorder) + i*(kMItemWidth + 1), mRect->top + kMenuBorder);
		Line(0,(kMItemHeight * kMenuHeight) + kMenuHeight);
	}
	// draw tiles:
	for (i=0; i<kNumberOfTiles; i++) {
		x = (kMItemWidth + 1) * (i % kMenuWidth) + 1 + mRect->left + kMenuBorder;
		y = (kMItemWidth + 1) * (i / kMenuWidth) + 1 + mRect->top + kMenuBorder;
		SetRect(&r,x,y,x+kMItemWidth,y+kMItemHeight);
		PlotTile(tiles,i*4,nil,&r);
	}
}

/* TileMenuItemHit
 * returns the item number clicked on; returns 0 if tile not clicked on.
 */
short TileMenuItemHit(Rect *mRect, Point hitPt)
{
	short item = 0;
	Rect r;
	
    // insure our hit pt is in the menu - not including the border!
	r = *mRect;
	InsetRect(&r,kMenuBorder,kMenuBorder);
	r.bottom -= 1; r.right -= 1; // kluge! the last grid lines!
	
	if (PtInRect( hitPt, &r)) {
	// To calc item (for an 8x8 square), we need to think in Octal: How deep down
	// they picked is the 8's column (hitPt.v); How far across they picked is the
	// 1s column (hitPt.h). While it's ok for the 8's col to be 0, the 1's column must
	// start at 1 and move on; the Menu Manager counts from 1 up, while the rest of
	// our C code counts from 0 up.			
	
	// 11/26/95 - should replace kMenuHeight w/ kMenuWidth??
	item = (((hitPt.v - (mRect->top + kMenuBorder)) / (kMItemHeight + 1 )) * kMenuHeight) +  // this can be 0
			((hitPt.h - (mRect->left + kMenuBorder)) / (kMItemWidth + 1 ) + 1); // this must start at 1!
	} // if (PtInRect( hitPt, &r))
	
	return item;
}

void TileMenuSize(short *width, short *height) 
{
	*width = kMItemWidth * kMenuWidth + (kMenuWidth + 1) + (2 * kMenuBorder);
	*height = kMItemHeight * kMenuHeight + (kMenuWidth + 1) + (2 * kMenuBorder);
}

// originally InvertItem

/* TMInvertItem
 * PREREQ: PORT MUST BE SET
 */
void TMInvertItem(Rect*mRect,short i)
{
	short x,y;
	Rect r, tempRect;
	
	x = (kMItemWidth + 1) * (i % kMenuWidth) + (mRect->left + kMenuBorder);
	y = (kMItemWidth + 1) * (i / kMenuWidth) + (mRect->top + kMenuBorder);
	SetRect(&r,x,y,x+kMItemWidth+2,y+kMItemHeight+2);
	TMInvertItem2(&r);
}

/* TMInvertItem2
 * PREREQ: PORT MUST BE SET
 */
void TMInvertItem2(Rect *itemRect)
{
	Rect r;
	RgnHandle rgn;	
	
	r = *itemRect; 
	rgn = NewRgn();
	OpenRgn();
	FrameRect(&r);
	InsetRect(&r,1,1);
	FrameRect(&r);
	CloseRgn(rgn);
	InvertRgn(rgn);
	DisposeRgn(rgn);
}



void SetTMTiles(GWorldPtr tiles)
{
	MenuHandle menu = GetMenu(mTiles);
	if (!menu) return;
	
	*(long *)(&(*menu)->menuData[(*menu)->menuData[0]+2]) = (long) tiles;
}

void CheckTMItem(short item)
{
	MenuHandle menu = GetMenu(mTiles);
	if (!menu) return;
	
	(*menu)->menuData[(*menu)->menuData[0]+1] = item; // set checked item.
}

short GetTMCheckedItem(void)
{
	MenuHandle menu = GetMenu(mTiles);
	if (!menu) return;
	
	return (short)(*menu)->menuData[(*menu)->menuData[0]+1]; // get checked item.
}