/*
 * WinManager.c
 *
 * Window Manager
 * 2/3/96
 */

/* Data Formats 2/16/96
 * gWinList
 *		The gWinList is a linked-list of windows registered in the WinManager; it is
 *		the central database for the module. It will contain two pieces of information
 *		about a window: the Window's WindowPtr, and a WMData structure. The WindowPtr
 *		is the lists Key, so information about a window can be found easily. The
 *		WMData structure contains important data about a window; including it's type,
 *		it's parent, and it's children.
 *
 * WMData.children
 *		Unlike the gWinList array above, the children lists contain only Key entries; no 
 *		special data is stored there. The children list stores a list of WindowPtrs
 * 		representing the children related to the window. 
 */

#include "WinManager.h"
#include "LinkedList.h"

WMData *FindWindowData(WindowPtr w);

LinkedListPtr gWinList = nil;

void RegisterWindow(WindowPtr w, WindowPtr parent, long t)
{
	WMData *data,*parentData;
	
	if (!w) return;
	if (!gWinList) NewLinkedList(&gWinList);
	
	// Create and Set WMData with function arguments.
	data = (WMData*)NewPtr(sizeof(WMData));
	if (!data) return;
	data->type = t;
	data->parent= parent;
	data->me = w;
	NewLinkedList(&(data->children)); // initializes as empty
	
	AddItemToList(gWinList, (long)data, (long)w);
	
	// Add child to the parent's child list.
	if (parent) {
		parentData = FindWindowData(parent);
		AddItemToList(parentData->children, 0, (long) w); 
	}
}

void UnregisterWindow(WindowPtr w)
{
	WMData *data,*parentData;

	if (!w) return;
	if (!gWinList) return;
	
	data = FindWindowData(w);
	if (data == nil) return; // window does not exist.

	// remove w from parent's child list
	if (data->parent) {
		parentData = FindWindowData(data->parent);
		RemoveItemFromList(parentData->children,(long)w);
	}
	
	// free WMData
	DisposeLinkedList(data->children);
	DisposePtr((Ptr)data);
	
	RemoveItemFromList(gWinList,(long)w);
}

// new, for WMData
WMData *FindWindowData(WindowPtr w)
{
	if (!w) return;
	if (!gWinList) return;
	
	return (WMData *)GetListItemByKey(gWinList,(long)w);
}

// Modified to call FindWindowData
long FindWindowType(WindowPtr w)
{
	if (!w) return;
	
	return (FindWindowData(w)->type);
}

long CountWindows(void)
{
	return LinkedListCount(gWinList);
}

WindowPtr FindNthWindow(long n)
{
	return (WindowPtr) GetListKeyByRank(gWinList,n);
}
void WMCloseWindow(WindowPtr w);// temp

void WMCloseWindow(WindowPtr w)
{
	WMData *data,parentData;
	long childCount,i;
	
	if (!w) return;
	if (!gWinList) return;

	data = FindWindowData(w);
	
	// close all children
	childCount = LinkedListCount(data->children);
	for (i=0; i<childCount; i++) {
		WMCloseWindow((WindowPtr)GetListKeyByRank(data->children,i+1)); // recursion
	}
	
	switch (data->type) {
	default:
		;
	}
}
